/* ************************************************************************
*   File: act.informative.c                             Part of CircleMUD *
*  Usage: Player-level commands of an informative nature                  *
*                                                                         *
*  All rights reserved.  See license.doc for complete information.        *
*                                                                         *
*  Copyright (C) 1993, 94 by the Trustees of the Johns Hopkins University *
*  CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.               *
************************************************************************ */

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"
#include "comm.h"
#include "interpreter.h"
#include "handler.h"
#include "db.h"
#include "spells.h"
#include "screen.h"
#include "constants.h"
#include "dg_scripts.h"
#include "introduction.h"
#include "boards.h"

/* extern variables */
extern int top_of_helpt;
extern struct help_index_element *help_table;
extern char *help;
extern struct time_info_data time_info;
extern struct index_data *obj_index;
extern char *credits;
extern char *news;
extern char *info;
extern char *motd;
extern char *imotd;
extern char *wizlist;
extern char *immlist;
extern char *policies;
extern char *handbook;
extern char *class_abbrevs[];
extern struct player_index_element *player_table;

/* extern functions */
ACMD(do_action);
ACMD(do_insult);
ACMD(do_oasis);
ACMD(do_gen_tog);
ACMD(do_wizutil);
ACMD(do_dgscripts);
ACMD(do_ban);
bitvector_t find_class_bitvector(const char *arg);
int level_exp(int chclass, int level, int race);
struct time_info_data *real_time_passed(time_t t2, time_t t1);
int compute_armor_class(struct char_data *ch);
long find_race_bitvector(char *arg);

/* local functions */
int sort_commands_helper(const void *a, const void *b);
void print_object_location(int num, struct obj_data *obj, struct char_data *ch, int recur);
void show_obj_to_char(struct obj_data *obj, struct char_data *ch, int mode);
void list_obj_to_char(struct obj_data *list, struct char_data *ch, int mode, int show);
void show_obj_modifiers(struct obj_data *obj, struct char_data *ch);
ACMD(do_look);
ACMD(do_examine);
ACMD(do_gold);
ACMD(do_score);
ACMD(do_inventory);
ACMD(do_equipment);
ACMD(do_time);
ACMD(do_weather);
ACMD(do_help);
ACMD(do_who);
ACMD(do_users);
ACMD(do_gen_ps);
void perform_mortal_where(struct char_data *ch, char *arg);
void perform_immort_where(struct char_data *ch, char *arg);
ACMD(do_where);
ACMD(do_levels);
ACMD(do_consider);
ACMD(do_diagnose);
ACMD(do_color);
ACMD(do_toggle);
void sort_commands(void);
ACMD(do_commands);
void diag_char_to_char(struct char_data *i, struct char_data *ch);
void look_at_char(struct char_data *i, struct char_data *ch);
void list_one_char(struct char_data *i, struct char_data *ch);
void list_char_to_char(struct char_data *list, struct char_data *ch);
void do_auto_exits(struct char_data *ch);
ACMD(do_exits);
void look_in_direction(struct char_data *ch, int dir);
void look_in_obj(struct char_data *ch, char *arg);
char *find_exdesc(char *word, struct extra_descr_data *list);
void look_at_target(struct char_data * ch, char *arg, int read);
void make_compass(struct char_data *ch, int i);

/* local globals */
int *cmd_sort_info;

/* For show_obj_to_char 'mode'.	/-- arbitrary */
#define SHOW_OBJ_LONG		0
#define SHOW_OBJ_SHORT		1
#define SHOW_OBJ_ACTION		2


void show_obj_to_char(struct obj_data *obj, struct char_data *ch, int mode)
{
  int found = 0;
  struct char_data *temp;
	
  if (!obj || !ch) {
    log("SYSERR: NULL pointer in show_obj_to_char(): obj=%p ch=%p", obj, ch);
    /*  SYSERR_DESC:
     *  Somehow a NULL pointer was sent to show_obj_to_char() in either the
     *  'obj' or the 'ch' variable.  The error will indicate which was NULL
     *  be listing both of the pointers passed to it.  This is often a
     *  difficult one to trace, and may require stepping through a debugger.
     */
    return;
  }
  
  if ((mode == 0) && obj->description) {
	if (!GET_OBJ_VAL(obj, 1) == 0 || OBJ_SAT_IN_BY(obj)) {
      temp = OBJ_SAT_IN_BY(obj);
      for (temp = OBJ_SAT_IN_BY(obj); temp; temp = NEXT_SITTING(temp)) {
		if (temp == ch)
		  found++;
		
      }
      if (found) {
		send_to_char(ch, "You are %s upon %s.", GET_POS(ch) == POS_SITTING ? "sitting" : 
	       "resting", obj->short_description);
	  goto end;
	  } 
	}
  }

  switch (mode) {
  case SHOW_OBJ_LONG:
    if (!IS_NPC(ch) && PRF_FLAGGED(ch, PRF_ROOMFLAGS))
     send_to_char(ch, "[%d] %s", GET_OBJ_VNUM(obj), obj->description);
    else
    send_to_char(ch, "%s", obj->description);
    break;

  case SHOW_OBJ_SHORT:
    send_to_char(ch, "%s", obj->short_description);
    break;

  case SHOW_OBJ_ACTION:
    switch (GET_OBJ_TYPE(obj)) {
    case ITEM_NOTE:
      if (obj->action_description) {
        char notebuf[MAX_NOTE_LENGTH + 64];

        snprintf(notebuf, sizeof(notebuf), "There is something written on it:\r\n\r\n%s", obj->action_description);
        page_string(ch->desc, notebuf, TRUE);
      } else
	send_to_char(ch, "It's blank.\r\n");
      return;

    case ITEM_DRINKCON:
      send_to_char(ch, "It looks like a drink container.");
      break;

    case ITEM_BOARD:
      Board_show_board(GET_OBJ_VNUM(obj), ch);
	  break;

    default:
      send_to_char(ch, "You see nothing special..");
      break;
    }
    break;

  default:
    log("SYSERR: Bad display mode (%d) in show_obj_to_char().", mode);
    /*  SYSERR_DESC:
     *  show_obj_to_char() has some predefined 'mode's (argument #3) to tell
     *  it what to display to the character when it is called.  If the mode
     *  is not one of these, it will output this error, and indicate what
     *  mode was passed to it.  To correct it, you will need to find the
     *  call with the incorrect mode and change it to an acceptable mode.
     */
    return;
  }
  
end:

  show_obj_modifiers(obj, ch);
  send_to_char(ch, "\r\n");
}


void show_obj_modifiers(struct obj_data *obj, struct char_data *ch)
{
  if (OBJ_FLAGGED(obj, ITEM_INVISIBLE))
    send_to_char(ch, " (invisible)");

  if (OBJ_FLAGGED(obj, ITEM_BLESS) && AFF_FLAGGED(ch, AFF_DETECT_ALIGN))
    send_to_char(ch, " ..It glows blue!");

  if (OBJ_FLAGGED(obj, ITEM_MAGIC) && AFF_FLAGGED(ch, AFF_DETECT_MAGIC))
    send_to_char(ch, " ..It glows yellow!");

  if (OBJ_FLAGGED(obj, ITEM_GLOW))
    send_to_char(ch, " ..It has a soft glowing aura!");

  if (OBJ_FLAGGED(obj, ITEM_HUM))
    send_to_char(ch, " ..It emits a faint humming sound!");
}


void list_obj_to_char(struct obj_data *list, struct char_data *ch, int mode, int show)
{
  struct obj_data *i;
  bool found = FALSE;

  for (i = list; i; i = i->next_content) {
    if (CAN_SEE_OBJ(ch, i)) {
      show_obj_to_char(i, ch, mode);
      found = TRUE;
    }
  }
  if (!found && show)
    send_to_char(ch, " Nothing.\r\n");
}


void diag_char_to_char(struct char_data *i, struct char_data *ch)
{
  struct {
    byte percent;
    const char *text;
  } diagnosis[] = {
    { 100, "is in excellent condition."			},
    {  90, "has a few scratches."			},
    {  75, "has some small wounds and bruises."		},
    {  50, "has quite a few wounds."			},
    {  30, "has some big nasty wounds and scratches."	},
    {  15, "looks pretty hurt."				},
    {   0, "is in awful condition."			},
    {  -1, "is bleeding awfully from big wounds."	},
  };
  int percent, ar_index;
  const char *pers = PERS(i, ch);

  if (GET_MAX_HIT(i) > 0)
    percent = (100 * GET_HIT(i)) / GET_MAX_HIT(i);
  else
    percent = -1;		/* How could MAX_HIT be < 1?? */

  for (ar_index = 0; diagnosis[ar_index].percent >= 0; ar_index++)
    if (percent >= diagnosis[ar_index].percent)
      break;

  send_to_char(ch, "%c%s %s\r\n", UPPER(*pers), pers + 1, diagnosis[ar_index].text);
}


void look_at_char(struct char_data *i, struct char_data *ch)
{
  int j, found;
  struct obj_data *tmp_obj;

  if (!ch->desc)
    return;

   if (i->player.description)
    send_to_char(ch, "%s", i->player.description);
  else
    act("You see nothing special about $m.", FALSE, i, 0, ch, TO_VICT);

  diag_char_to_char(i, ch);

  found = FALSE;
  for (j = 0; !found && j < NUM_WEARS; j++)
    if (GET_EQ(i, j) && CAN_SEE_OBJ(ch, GET_EQ(i, j)))
      found = TRUE;

  if (found) {
    send_to_char(ch, "\r\n");	/* act() does capitalization. */
    act("$n is using:", FALSE, i, 0, ch, TO_VICT);
    for (j = 0; j < NUM_WEARS; j++)
      if (GET_EQ(i, j) && CAN_SEE_OBJ(ch, GET_EQ(i, j))) {
	send_to_char(ch, "%s", wear_where[j]);
	show_obj_to_char(GET_EQ(i, j), ch, SHOW_OBJ_SHORT);
      }
  }
  if (ch != i && (IS_THIEF(ch) || GET_LEVEL(ch) >= LVL_IMMORT)) {
    found = FALSE;
    act("\r\nYou attempt to peek at $s inventory:", FALSE, i, 0, ch, TO_VICT);
    for (tmp_obj = i->carrying; tmp_obj; tmp_obj = tmp_obj->next_content) {
      if (CAN_SEE_OBJ(ch, tmp_obj) && (rand_number(0, 20) < GET_LEVEL(ch))) {
	show_obj_to_char(tmp_obj, ch, SHOW_OBJ_SHORT);
	found = TRUE;
      }
    }

    if (!found)
      send_to_char(ch, "You can't see anything.\r\n");
  }
}


void list_one_char(struct char_data *i, struct char_data *ch)
{
  struct obj_data *chair;

  const char *positions[] = {
    " is lying here, dead.",
    " is lying here, mortally wounded.",
    " is lying here, incapacitated.",
    " is lying here, stunned.",
    " is sleeping here.",
    " is resting here.",
    " is sitting here.",
    "!FIGHTING!",
    " is standing here."
  };

  if (!IS_NPC(ch) && PRF_FLAGGED(ch, PRF_ROOMFLAGS) && IS_NPC(i)) 
     send_to_char(ch,"[%d] ", GET_MOB_VNUM(i));
  
  if (IS_NPC(i) && i->player.long_descr && GET_POS(i) == GET_DEFAULT_POS(i)) {
    if (AFF_FLAGGED(i, AFF_INVISIBLE))
      send_to_char(ch, "*");

    if (AFF_FLAGGED(ch, AFF_DETECT_ALIGN)) {
      if (IS_EVIL(i))
	send_to_char(ch, "(Red Aura) ");
      else if (IS_GOOD(i))
	send_to_char(ch, "(Blue Aura) ");
    }
    send_to_char(ch, "%s", i->player.long_descr);

    if (AFF_FLAGGED(i, AFF_SANCTUARY))
      act("...$e glows with a bright light!", FALSE, i, 0, ch, TO_VICT);
    if (AFF_FLAGGED(i, AFF_BLIND))
      act("...$e is groping around blindly!", FALSE, i, 0, ch, TO_VICT);

    return;
  }

  if (knows_char(ch, i) && !IS_NPC(i)) {
    send_to_char(ch, "%s", i->player.name);
    if (!PRF_FLAGGED(i, PRF_NOTITLESET))
	  send_to_char(ch, " %s&n", GET_TITLE(i));
  } else if (!IS_NPC(i)) {
    char buf[64];

    sprintf(buf, "%s", GET_FEATURE1(i));

    send_to_char(ch, "%s, %s %s %s", CAP(buf), GET_FEATURE2(i), pc_race_types_type2[(int)GET_RACE(i)], 
    show_sex(i));
  }

  if (AFF_FLAGGED(i, AFF_INVISIBLE))
    send_to_char(ch, " (invisible)");
  if (AFF_FLAGGED(i, AFF_HIDE))
    send_to_char(ch, " (hidden)");
  if (!IS_NPC(i) && !i->desc)
    send_to_char(ch, " (linkless)");
  if (!IS_NPC(i) && PLR_FLAGGED(i, PLR_WRITING))
    send_to_char(ch, " (writing)");
  if (!IS_NPC(i) && PRF_FLAGGED(i, PRF_BUILDWALK))
    send_to_char(ch, " (buildwalk)");

  if (GET_POS(i) != POS_FIGHTING) {
    if (!SITTING(i)){
	  send_to_char(ch, "%s", positions[(int) GET_POS(i)]);
	} else {
      chair = SITTING(i);
      send_to_char(ch, " is %s upon %s.", 
             ((GET_POS(i) == POS_SITTING) ? "sitting" : "resting"),
             (CAN_SEE_OBJ(ch, chair) ? chair->short_description : "something"));
    }
  } else {
    if (FIGHTING(i)) {
      send_to_char(ch, " is here, fighting ");
      if (FIGHTING(i) == ch)
	send_to_char(ch, "YOU!");
      else {
	if (IN_ROOM(i) == IN_ROOM(FIGHTING(i)))
	  send_to_char(ch, "%s!", PERS(FIGHTING(i), ch));
	else
	  send_to_char(ch,  "someone who has already left!");
      }
    } else			/* NIL fighting pointer */
      send_to_char(ch, " is here struggling with thin air.");
  }

  if (AFF_FLAGGED(ch, AFF_DETECT_ALIGN)) {
    if (IS_EVIL(i))
      send_to_char(ch, " (Red Aura)");
    else if (IS_GOOD(i))
      send_to_char(ch, " (Blue Aura)");
  }
  send_to_char(ch, "\r\n");

  if (AFF_FLAGGED(i, AFF_SANCTUARY))
    act("...$e glows with a bright light!", FALSE, i, 0, ch, TO_VICT);
}



void list_char_to_char(struct char_data *list, struct char_data *ch)
{
  struct char_data *i;

  for (i = list; i; i = i->next_in_room)
    if (ch != i) {
      if (CAN_SEE(ch, i))
	list_one_char(i, ch);
      else if (IS_DARK(IN_ROOM(ch)) && !CAN_SEE_IN_DARK(ch) &&
	       AFF_FLAGGED(i, AFF_INFRAVISION))
	send_to_char(ch, "You see a pair of glowing red eyes looking your way.\r\n");
    }
}


void do_auto_exits(struct char_data *ch)
{
  int door, slen = 0;

  send_to_char(ch, "%s[ Exits: ", CCCYN(ch, C_NRM));

  for (door = 0; door < NUM_OF_DIRS; door++) {
    if (!EXIT(ch, door) || EXIT(ch, door)->to_room == NOWHERE)
      continue;
    if (EXIT_FLAGGED(EXIT(ch, door), EX_CLOSED))
      continue;

    send_to_char(ch, "%c ", LOWER(*dirs[door]));
    slen++;
  }

  send_to_char(ch, "%s]%s\r\n", slen ? "" : "None!", CCNRM(ch, C_NRM));
}


ACMD(do_exits)
{
  int door, len = 0;

  if (AFF_FLAGGED(ch, AFF_BLIND)) {
    send_to_char(ch, "You can't see a damned thing, you're blind!\r\n");
    return;
  }

  send_to_char(ch, "Obvious exits:\r\n");

  for (door = 0; door < NUM_OF_DIRS; door++) {
    if (!EXIT(ch, door) || EXIT(ch, door)->to_room == NOWHERE)
      continue;
    if (EXIT_FLAGGED(EXIT(ch, door), EX_CLOSED))
      continue;

    len++;

    if (GET_LEVEL(ch) >= LVL_IMMORT)
      send_to_char(ch, "%-5s - [%5d] %s\r\n", dirs[door], GET_ROOM_VNUM(EXIT(ch, door)->to_room),
		world[EXIT(ch, door)->to_room].name);
    else
      send_to_char(ch, "%-5s - %s\r\n", dirs[door], IS_DARK(EXIT(ch, door)->to_room) &&
		!CAN_SEE_IN_DARK(ch) ? "Too dark to tell." : world[EXIT(ch, door)->to_room].name);
  }

  if (!len)
    send_to_char(ch, " None.\r\n");
}



void look_at_room(struct char_data *ch, int ignore_brief)
{
  char *blood_messages[] = {
    "Should never see this.",
    "There are some drops of blood here.",
    "There's a little blood here.",
    "You're standing in some blood.",
    "There's quite some blood on the floor.",
    "The blood in this room is flowing.",
    "There's so much blood here it's intoxicating!",
    "How much more blood can there be in any one room?",
    "Such carnage. The God of Death is feastin' tonight!",
    "You are splashing around in the blood of the slain!",
    "Even the walls are revolted by the death and destruction!",
    "The Gods should show some mercy and cleanse this horrid room!",
    "So much blood has been shed here, you are drowning in it!",
    "\n"
  };
	
  if (!ch->desc)
    return;
  
  if (ROOM_AFFECTED(IN_ROOM(ch), RAFF_PERMDARK)) {
	if (GET_LEVEL(ch) < LVL_IMMORT) {
	  send_to_char(ch, "It is pitch black...\r\n");
	  return;
	}
  } else if (IS_DARK(IN_ROOM(ch)) && !CAN_SEE_IN_DARK(ch)) {
    send_to_char(ch, "It is pitch black...\r\n");
    return;
  } else if (AFF_FLAGGED(ch, AFF_BLIND)) {
    send_to_char(ch, "You see nothing but infinite darkness...\r\n");
    return;
  } else if (ROOM_AFFECTED(IN_ROOM(ch), RAFF_FOG) && (!AFF_FLAGGED(ch, AFF_INFRAVISION))) {
    if (GET_LEVEL(ch) < LVL_IMMORT) {
	  send_to_char(ch, "Your view is obscured by a thick fog...\r\n");
	  return;
	}
  }

  send_to_char(ch, "%s", CCCYN(ch, C_NRM));
  if (!IS_NPC(ch) && PRF_FLAGGED(ch, PRF_ROOMFLAGS)) {
    char buf[MAX_STRING_LENGTH];

    sprintbitarray(ROOM_FLAGS(IN_ROOM(ch)), room_bits, RF_ARRAY_MAX, buf);

	if (BRIDGE(IN_ROOM(ch)) > 0) {
	  send_to_char(ch, "[%5d] %s [ %s] [ %s ]", GET_ROOM_VNUM(IN_ROOM(ch)), world[IN_ROOM(ch)].name, buf, bridges[BRIDGE(IN_ROOM(ch))]);
    } else
      send_to_char(ch, "[%5d] %s [ %s]", GET_ROOM_VNUM(IN_ROOM(ch)), world[IN_ROOM(ch)].name, buf);
  
    sprinttype(ROOM_AFF(IN_ROOM(ch)), room_affections, buf, sizeof(buf));
    if (ROOM_AFF(IN_ROOM(ch)) > 0) 
	  send_to_char(ch, " [ %s ]", buf);

  } else
    send_to_char(ch, "%s", world[IN_ROOM(ch)].name);

  send_to_char(ch, "%s\r\n", CCNRM(ch, C_NRM));

  if ((!IS_NPC(ch) && !PRF_FLAGGED(ch, PRF_BRIEF)) || ignore_brief ||
      ROOM_FLAGGED(IN_ROOM(ch), ROOM_DEATH)) {
	char buf[MAX_STRING_LENGTH], buf2[MAX_STRING_LENGTH];
	int j = 0;

    prepare_text(world[IN_ROOM(ch)].description, buf);
	if (PRF_FLAGGED(ch, PRF_COMPASS)) {
	  int i = 0;
      do {
	    if (i < 5) {
		  make_compass(ch, i);
		  j = get_next_line(buf, buf2, j, 74);
		  if (!j)
			strcat(buf2, "\r\n");
		  i++;
		} else
		  j = get_next_line(buf, buf2, j, 80);

		if (j) 
		  strcat(buf2, "\r\n");
    	send_to_char(ch, buf2);
	  } while (j != 0);
	  if (i < 5) {
		while (i < 5) {
		  make_compass(ch, i);
		  send_to_char(ch, "\r\n");
		  i++;
		}
	  }
	} else { 
	  do {
        j = get_next_line(buf, buf2, j, 80);
	    if (j)
		  strcat(buf2, "\r\n");
    	
		send_to_char(ch, buf2);
	  } while (j != 0);
	}
  }

  if (RM_BLOOD(IN_ROOM(ch) > 0)) 
	send_to_char(ch,"&R%s&n\r\n", blood_messages[RM_BLOOD(IN_ROOM(ch))]);

  /* autoexits */
  if (!IS_NPC(ch) && PRF_FLAGGED(ch, PRF_AUTOEXIT))
    do_auto_exits(ch);

  /* now list characters & objects */
  send_to_char(ch, "%s", CCGRN(ch, C_NRM));
  list_obj_to_char(world[IN_ROOM(ch)].contents, ch, SHOW_OBJ_LONG, FALSE);
  send_to_char(ch, "%s", CCYEL(ch, C_NRM));
  list_char_to_char(world[IN_ROOM(ch)].people, ch);
  send_to_char(ch, "%s", CCNRM(ch, C_NRM));
}

void make_compass(struct char_data *ch, int i)
{
  switch (i) {
  case 0:
    send_to_char(ch, "&y+---+&n ");
	break;
  case 1:
	send_to_char(ch, "&y|&n %s &y|&n ", EXIT(ch, NORTH) ? EXIT(ch, NORTH)->to_room == NOWHERE ? " " : "|" : " ");
	break;
  case 2: 
	send_to_char(ch, "&y|&n%s", EXIT(ch, WEST) ? EXIT(ch, WEST)->to_room == NOWHERE ? " " : "-" : " ");
    if (EXIT(ch, DOWN) && !(EXIT(ch, DOWN)->to_room == NOWHERE))
	  if (EXIT(ch, UP) && !(EXIT(ch, UP)->to_room == NOWHERE))
	    send_to_char(ch, "*");
	  else 
	    send_to_char(ch, "D");
    else if (EXIT(ch, UP) && !(EXIT(ch, UP)->to_room == NOWHERE))
	  send_to_char(ch, "U");
	else 
	  send_to_char(ch, "o");
	send_to_char(ch, "%s&y|&n ", EXIT(ch, EAST) ? EXIT(ch, EAST)->to_room == NOWHERE ? " " : "-" : " ");
	break;
  case 3:
	send_to_char(ch, "&y|&n %s &y|&n ", EXIT(ch, SOUTH) ? "|" : " ");
	break;
  case 4:
    send_to_char(ch, "&y+---+&n ");
	break;
  default: 
	log("SYSYERR: Default case reached in make_compass()!");
	break;
  }
}


void look_in_direction(struct char_data *ch, int dir)
{
  if (EXIT(ch, dir)) {
    if (EXIT(ch, dir)->general_description)
      send_to_char(ch, "%s", EXIT(ch, dir)->general_description);
    else
      send_to_char(ch, "You see nothing special.\r\n");

    if (EXIT_FLAGGED(EXIT(ch, dir), EX_CLOSED) && EXIT(ch, dir)->keyword)
      send_to_char(ch, "The %s is closed.\r\n", fname(EXIT(ch, dir)->keyword));
    else if (EXIT_FLAGGED(EXIT(ch, dir), EX_ISDOOR) && EXIT(ch, dir)->keyword)
      send_to_char(ch, "The %s is open.\r\n", fname(EXIT(ch, dir)->keyword));
  } else
    send_to_char(ch, "Nothing special there...\r\n");
}



void look_in_obj(struct char_data *ch, char *arg)
{
  struct obj_data *obj = NULL;
  struct char_data *dummy = NULL;
  int amt, bits;

  if (!*arg)
    send_to_char(ch, "Look in what?\r\n");
  else if (!(bits = generic_find(arg, FIND_OBJ_INV | FIND_OBJ_ROOM |
				 FIND_OBJ_EQUIP, ch, &dummy, &obj))) {
    send_to_char(ch, "There doesn't seem to be %s %s here.\r\n", AN(arg), arg);
  } else if ((GET_OBJ_TYPE(obj) != ITEM_DRINKCON) &&
	     (GET_OBJ_TYPE(obj) != ITEM_FOUNTAIN) &&
	     (GET_OBJ_TYPE(obj) != ITEM_CONTAINER))
    send_to_char(ch, "There's nothing inside that!\r\n");
  else {
    if (GET_OBJ_TYPE(obj) == ITEM_CONTAINER) {
      if (OBJVAL_FLAGGED(obj, CONT_CLOSED))
	send_to_char(ch, "It is closed.\r\n");
      else {
	send_to_char(ch, "%s", fname(obj->name));
	switch (bits) {
	case FIND_OBJ_INV:
	  send_to_char(ch, " (carried): \r\n");
	  break;
	case FIND_OBJ_ROOM:
	  send_to_char(ch, " (here): \r\n");
	  break;
	case FIND_OBJ_EQUIP:
	  send_to_char(ch, " (used): \r\n");
	  break;
	}

	list_obj_to_char(obj->contains, ch, SHOW_OBJ_SHORT, TRUE);
      }
    } else {		/* item must be a fountain or drink container */
      if (GET_OBJ_VAL(obj, 1) <= 0)
	send_to_char(ch, "It is empty.\r\n");
      else {
	if (GET_OBJ_VAL(obj,0) <= 0 || GET_OBJ_VAL(obj,1)>GET_OBJ_VAL(obj,0)) {
	  send_to_char(ch, "Its contents seem somewhat murky.\r\n"); /* BUG */
	} else {
          char buf2[MAX_STRING_LENGTH];
	  amt = (GET_OBJ_VAL(obj, 1) * 3) / GET_OBJ_VAL(obj, 0);
	  sprinttype(GET_OBJ_VAL(obj, 2), color_liquid, buf2, sizeof(buf2));
	  send_to_char(ch, "It's %sfull of a %s liquid.\r\n", fullness[amt], buf2);
	}
      }
    }
  }
}



char *find_exdesc(char *word, struct extra_descr_data *list)
{
  struct extra_descr_data *i;

  for (i = list; i; i = i->next)
    if (isname(word, i->keyword))
      return (i->description);

  return (NULL);
}


void look_at_target(struct char_data * ch, char *arg, int read)
{
  int bits, found = FALSE, j, fnum, i = 0;
  struct char_data *found_char = NULL;
  struct obj_data *obj, *found_obj = NULL;
  char *desc;

  if (!ch->desc)
    return;

  if (!*arg) {
    send_to_char(ch, "Look at what?\r\n");
    return;
  }
  
  for (obj = ch->carrying; obj;obj=obj->next_content) {
    if (GET_OBJ_TYPE(obj) == ITEM_BOARD) { /* Board in inventory? */
      found = TRUE; /* Yes */
      break;
	}
  }
  
  if (!obj) {
    for (obj = world[ch->in_room].contents; obj;obj=obj->next_content) {
      if (GET_OBJ_TYPE(obj) == ITEM_BOARD) { /* Board in room? */
        found = TRUE; /* Yes */
        break;
	  }
	}
  }
  
  if (found) { /* Let's go down the list of possible actions: */
    skip_spaces(&arg);
	if (!*arg) /* No argument */
      send_to_char(ch, "Read what?\r\n");
    else if (isname(arg, obj->name)) {
	  Board_show_board(GET_OBJ_VNUM(obj), ch);
	  return;
    } else if (is_number(arg)) {
      Board_display_msg(GET_OBJ_VNUM(obj), ch, atoi(arg));
	  return;
	}
	/* Probably a "read mail" */
  }
 
  found = FALSE;
  bits = generic_find(arg, FIND_OBJ_INV | FIND_OBJ_ROOM | FIND_OBJ_EQUIP |
		      FIND_CHAR_ROOM, ch, &found_char, &found_obj);

  /* Is the target a character? */
  if (found_char != NULL) {
    look_at_char(found_char, ch);
    if (ch != found_char) {
      if (CAN_SEE(found_char, ch))
	act("$n looks at you.", TRUE, ch, 0, found_char, TO_VICT);
      act("$n looks at $N.", TRUE, ch, 0, found_char, TO_NOTVICT);
    }
    return;
  }

  /* Strip off "number." from 2.foo and friends. */
  if (!(fnum = get_number(&arg))) {
    send_to_char(ch, "Look at what?\r\n");
    return;
  }
 
  /* Does the argument match an extra desc in the room? */
  if ((desc = find_exdesc(arg, world[IN_ROOM(ch)].ex_description)) != NULL && ++i == fnum) {
    page_string(ch->desc, desc, FALSE);
    return;
  }
  /* Does the argument match an extra desc in the char's equipment? */
  for (j = 0; j < NUM_WEARS && !found; j++)
    if (GET_EQ(ch, j) && CAN_SEE_OBJ(ch, GET_EQ(ch, j)))
      if ((desc = find_exdesc(arg, world[ch->in_room].ex_description)) != NULL && ++i == fnum) {
	send_to_char(ch, "%s", desc);
	found = TRUE;
      }
  /* Does the argument match an extra desc in the char's inventory? */
  for (obj = ch->carrying; obj && !found; obj = obj->next_content) {
    if (CAN_SEE_OBJ(ch, obj))
      if ((desc = find_exdesc(arg, obj->ex_description)) != NULL && ++i == fnum) {
		if (GET_OBJ_TYPE(obj) == ITEM_BOARD) {
          Board_show_board(GET_OBJ_VNUM(obj), ch);
		  return;
		} else
 	      send_to_char(ch, "%s", desc);
        found = TRUE;
	  }
  }

  /* Does the argument match an extra desc of an object in the room? */
  for (obj = world[IN_ROOM(ch)].contents; obj && !found; obj = obj->next_content)
    if (CAN_SEE_OBJ(ch, obj))
      if ((desc = find_exdesc(arg, obj->ex_description)) != NULL && ++i == fnum) {
		if(GET_OBJ_TYPE(obj) == ITEM_BOARD) {
          Board_show_board(GET_OBJ_VNUM(obj), ch);
		  return;
		} else
	  send_to_char(ch, "%s", desc);
    found = TRUE;
      }

  /* If an object was found back in generic_find */
  if (bits) {
    if (!found)
      show_obj_to_char(found_obj, ch, SHOW_OBJ_ACTION);
    else {
      show_obj_modifiers(found_obj, ch);
      send_to_char(ch, "\r\n");
    }
  } else if (!found)
    send_to_char(ch, "You do not see that here.\r\n");
}   


ACMD(do_look)
{
  int look_type;
 
  if (!ch->desc)
    return;

  if (GET_POS(ch) < POS_SLEEPING)
    send_to_char(ch, "You can't see anything but stars!\r\n");
  else if (AFF_FLAGGED(ch, AFF_BLIND))
    send_to_char(ch, "You can't see a damned thing, you're blind!\r\n");
  else if (IS_DARK(IN_ROOM(ch)) && !CAN_SEE_IN_DARK(ch)) {
    send_to_char(ch, "It is pitch black...\r\n");
    list_char_to_char(world[IN_ROOM(ch)].people, ch);	/* glowing red eyes */
  } else {
    char arg[MAX_INPUT_LENGTH], arg2[MAX_INPUT_LENGTH];

    half_chop(argument, arg, arg2);

    if (subcmd == SCMD_READ) {
      if (!*arg)
	send_to_char(ch, "Read what?\r\n");
      else
	look_at_target(ch, arg, 1);
      return;
    }
    if (!*arg)			/* "look" alone, without an argument at all */
      look_at_room(ch, 1);
    else if (is_abbrev(arg, "in"))
      look_in_obj(ch, arg2);
    /* did the char type 'look <direction>?' */
    else if ((look_type = search_block(arg, dirs, FALSE)) >= 0)
      look_in_direction(ch, look_type);
    else if (is_abbrev(arg, "at"))
      look_at_target(ch, arg2, 0);
    else
      look_at_target(ch, arg, 0);
  }
}



ACMD(do_examine)
{
  struct char_data *tmp_char;
  struct obj_data *tmp_object;
  char tempsave[MAX_INPUT_LENGTH], arg[MAX_INPUT_LENGTH];

  one_argument(argument, arg);

  if (!*arg) {
    send_to_char(ch, "Examine what?\r\n");
    return;
  }

  /* look_at_target() eats the number. */
  look_at_target(ch, strcpy(tempsave, arg), 0);

  generic_find(arg, FIND_OBJ_INV | FIND_OBJ_ROOM | FIND_CHAR_ROOM |
		      FIND_OBJ_EQUIP, ch, &tmp_char, &tmp_object);

  if (tmp_object) {
    if ((GET_OBJ_TYPE(tmp_object) == ITEM_DRINKCON) ||
	(GET_OBJ_TYPE(tmp_object) == ITEM_FOUNTAIN) ||
	(GET_OBJ_TYPE(tmp_object) == ITEM_CONTAINER)) {
      send_to_char(ch, "When you look inside, you see:\r\n");
      look_in_obj(ch, arg);
    }
  }
}



ACMD(do_gold)
{
  if (GET_GOLD(ch) == 0)
    send_to_char(ch, "You're broke!\r\n");
  else if (GET_GOLD(ch) == 1)
    send_to_char(ch, "You have one miserable little gold coin.\r\n");
  else
    send_to_char(ch, "You have %d gold coins.\r\n", GET_GOLD(ch));
}


char *return_skill(int skill)
{
  switch(skill) {
  case 0: return ("awful");
  case 1: return ("awful");
  case 2: return ("awful");
  case 3: return ("awful");
  case 4: return ("awful");
  case 5: return ("awful");
  case 6: return ("awful");
  case 7: return ("bad");
  case 8: return ("bad");
  case 9: return ("bad");
  case 10: return ("poor");
  case 11: return ("average");
  case 12: return ("fair");
  case 13: return ("good");
  case 14: return ("very good");
  case 15: return ("outstanding");
  case 16: return ("superb");
  case 17: return ("excellent");
  case 18: return ("godlike");
  case 19: return ("godlike");
  case 20: return ("godlike");
  case 21: return ("godlike");
  case 22: return ("godlike");
  case 23: return ("godlike");
  case 24: return ("godlike");
  case 25: return ("godlike");
  }

  log("SYSERR: Reached default case in return_skill()");
  return ("unknown");
}

char *return_armor(int armor)
{
  static char buf[128];

  if ((armor) >= 35)
	strcpy(buf, "You are butt-naked! Put some clothes on, you pervert!");
  else if ((armor) >= 25)
	strcpy(buf, "Your armor could barely stop a needle!");
  else if ((armor) >= 15)
	strcpy(buf, "You are somewhat armored.");
  else if ((armor) >= 5)
	strcpy(buf, "You are armored.");
  else if ((armor) >= 0)
	strcpy(buf, "You are well armored");
  else if ((armor) >= -5)
	strcpy(buf, "You are very well armored.");
  else if ((armor) >= -15)
	strcpy(buf, "You are a walking tank!");
  else if ((armor) >= -25)
	strcpy(buf, "You have incredible armor.");
  else if ((armor) >= -35)
	strcpy(buf, "You are impermeable!");
  else if ((armor) >= -100)
	strcpy(buf, "You have Godlike armor!");
  else {
	log("SYSERR: Reached default case in return_armor()");
	strcpy(buf, "Error, please report");
  }

  sprintf(buf, "%s (%d <-> 10)", buf, armor);
  return(buf);
}

char *return_alignment(int alignment)
{
  if (alignment >= 990)
    return("You are Good in person!");
  else if (alignment >= 900)
	return("The ArchAngels are jealous of you!");
  else if (alignment >= 800)
	return("The ArchAngels welcome you in their ranks of Goodness.");
  else if (alignment >= 700)
	return("You are nearly an ArchAngel.");
  else if (alignment >= 600)
	return("You are an Angel.");
  else if (alignment >= 500)
	return("You are a Saint.");
  else if (alignment >= 400)
	return("You are an Arch Bisshop.");
  else if (alignment >= 300)
	return("You are a Bisshop.");
  else if (alignment >= 200)
	return("You are a Patriarch");
  else if (alignment >= 150)
	return("You are a Benefactor.");
  else if (alignment >= 100)
	return("You are Good.");
  else if (alignment >= 50)
	return("You are slightly heading towards Good.");
  else if (alignment >= 25)
	return("You are neither Good nor Evil.");
  else if (alignment >= 0 && alignment <= 0)
	return("You are truely neutral.");
  else if (alignment >= -25)
	return("You are neither Good nor Evil.");
  else if (alignment >= -50)
	return("You are slightly heading towards Evil.");
  else if (alignment >= -100)
	return("You are Evil.");
  else if (alignment >= -200)
	return("You are a Bandit.");
  else if (alignment >= -300)
	return("You are a Priest of Evil.");
  else if (alignment >= -400)
	return("You are ArchEvil.");
  else if (alignment >= -500)
	return("You are a Warrior of the reign of Chaos.");
  else if (alignment >= -600)
	return("You are a Elite of Evil.");
  else if (alignment >= -700)
	return("You are the Pioneer of Evil");
  else if (alignment >= -800)
	return("You are a true Deamon.");
  else if (alignment >= -900)
	return("The Demons of the 7th Hell are jealous of you.");
  else if (alignment >= -990)
	return("You are approaching the Devil!");
  else if (alignment >= -1000)
	return("You are the Devil himself!");
  else { 
	log("SYSERR: Reached default case in return_alignment()");
	return("Error, please report");
  }
}

char *return_experience(struct char_data *ch)
{
  static char buf[128] = "";

  if (GET_LEVEL(ch) < LVL_IMMORT) 
    sprintf(buf, "You scored %d experience, %d until next level.", GET_EXP(ch), 
		level_exp(GET_CLASS(ch), GET_LEVEL(ch) + 1, GET_RACE(ch)) - GET_EXP(ch));
  else 
	sprintf(buf, "As you are immortal, you don't need any more experience.");

  return(buf);
}

char *doscore_end(struct char_data *ch)
{
  static char buf[MAX_STRING_LENGTH];

  switch (GET_POS(ch)) {
  case POS_DEAD:
    strcpy(buf, "  ( You are DEAD!                                                        )\r\n");
    break;
  case POS_MORTALLYW:
    strcpy(buf, "  ( You are mortally wounded!  You should seek help!                     )\r\n");
    break;
  case POS_INCAP:
    strcpy(buf, "  ( You are incapacitated, slowly fading away...                         )\r\n");
    break;
  case POS_STUNNED:
    strcpy(buf, "  ( You are stunned!  You can't move!                                    )\r\n");
    break;
  case POS_SLEEPING: 
    strcpy(buf, "  ( You are sleeping.                                                    )\r\n");
    break;
  case POS_RESTING:
    strcpy(buf, "  ( You are resting.                                                     )\r\n");
    break;
  case POS_SITTING:
    if (!SITTING(ch))
	  strcpy(buf, "  ( You are sitting.                                                     )\r\n");
    else { 
      struct obj_data *chair = SITTING(ch);
      sprintf(buf, "  ( You are sitting upon %-10s                                      )\r\n", chair->short_description);
    }
	break;
  case POS_FIGHTING:
    sprintf(buf, "  ( You are fighting %-20s                            )\r\n", FIGHTING(ch) ? PERS(FIGHTING(ch), ch) : "thin air");
    break;
  case POS_STANDING:
    strcpy(buf, "  ( You are standing.                                                    )\r\n");
    break;
  default:
    strcpy(buf, "  ( You are floating.                                                    )\r\n");
    break;
  }

  if (GET_COND(ch, DRUNK) > 10)
    sprintf(buf, "%s  ( You are intoxicated.                                                 )\r\n", buf);

  if (GET_COND(ch, FULL) == 0)
    sprintf(buf, "%s  ( You are hungry.                                                      )\r\n", buf);

  if (GET_COND(ch, THIRST) == 0)
    sprintf(buf, "%s  ( You are thirsty.                                                     )\r\n", buf);

  if (AFF_FLAGGED(ch, AFF_BLIND))
    sprintf(buf, "%s  ( You have been blinded!                                               )\r\n", buf);

  if (AFF_FLAGGED(ch, AFF_INVISIBLE))
    sprintf(buf, "%s  ( You are invisible.                                                   )\r\n", buf);

  if (AFF_FLAGGED(ch, AFF_DETECT_INVIS))
    sprintf(buf, "%s  ( You are sensitive to the presence of invisible things.               )\r\n", buf);

  if (AFF_FLAGGED(ch, AFF_SANCTUARY))
    sprintf(buf, "%s  ( You are protected by Sanctuary.                                      )\r\n", buf);

  if (AFF_FLAGGED(ch, AFF_POISON))
    sprintf(buf, "%s  ( You are poisoned!                                                    )\r\n", buf);

  if (AFF_FLAGGED(ch, AFF_CHARM))
    sprintf(buf, "%s  ( You have been charmed!                                               )\r\n", buf);

  if (affected_by_spell(ch, SPELL_ARMOR))
    sprintf(buf, "%s  ( You feel protected.                                                  )\r\n", buf);

  if (AFF_FLAGGED(ch, AFF_INFRAVISION))
    sprintf(buf, "%s  ( Your eyes are glowing red.                                           )\r\n", buf);

  if (PRF_FLAGGED(ch, PRF_SUMMONABLE))
    sprintf(buf, "%s  ( You are summonable by other players.                                 )\r\n", buf);

  return(buf);
}
  
ACMD(do_score)
{
  if (IS_NPC(ch))
    return;
  
  send_to_char(ch, "   .=~-:|:-~-:|:-~-:|:-~-:|:-~-:|:-[status]-:|:-~-:|:-~-:|:-~-:|:-~-:|:=.\r\n"
	               "  (                                                                      )\r\n"
				   "  ( Name:  %-19s  Level: %-3d  Sex: %-6s  Age: %d         )\r\n"
				   "  (                                                                      )\r\n"
				   "  ( Strength    : %-11s   Hitpoints: %-4d/%-4d   Regen: %-4d       )\r\n"
				   "  ( Dexterity   : %-11s   Movement : %-4d/%-4d   Regen: %-4d       )\r\n"
				   "  ( Constitution: %-11s   Mana     : %-4d/%-4d   Regen: %-4d       )\r\n" 
				   "  ( Intelligence: %-11s                                            )\r\n" 
				   "  ( Wisdom      : %-11s   Race : %-10s      Class: %-10s )\r\n"
				   "  (                                                                      )\r\n"
                   "  (   -:|:-~-:|:-~-:|:-~-:|:-~-:|:--:|:-~-:|:-~-:|:-~-:|:-~-:|:-~-:|:-   )\r\n"
                   "  (                                                                      )\r\n"
	               "  ( %-68s )\r\n"
				   "  ( %-68s )\r\n"
				   "  (                                                                      )\r\n"
				   "  ( You are carrying %-3d items, weighing %-3d pounds. (%-4d max)          )\r\n"
				   "  (                                                                      )\r\n"
				   "  (   -:|:-~-:|:-~-:|:-~-:|:-~-:|:--:|:-~-:|:-~-:|:-~-:|:-~-:|:-~-:|:-   )\r\n"
				   "  (                                                                      )\r\n"
				   "  ( %-68s )\r\n"
				   "  ( You've died %-4d times.                                              )\r\n"
				   "%s"
				   "  (                                                                      )\r\n"
				   "   `=~-:|:-~-:|:-~-:|:-~-:|:-~-:|:--:|:-~-:|:-~-:|:-~-:|:-~-:|:-~-:|:-~=`\r\n",

				   GET_NAME(ch), GET_LEVEL(ch), GET_SEX(ch) ? (GET_SEX(ch)==SEX_MALE ? "Male":"Female") :"None", 
				   GET_AGE(ch), return_skill(GET_STR(ch)), GET_HIT(ch), GET_MAX_HIT(ch), hit_gain(ch),
				   return_skill(GET_DEX(ch)), GET_MOVE(ch), GET_MAX_MOVE(ch), move_gain(ch),
				   return_skill(GET_CON(ch)), GET_MANA(ch), GET_MAX_MANA(ch), mana_gain(ch),
				   return_skill(GET_INT(ch)), return_skill(GET_WIS(ch)), pc_race_types[GET_RACE(ch)],
				   pc_class_types[GET_CLASS(ch)], return_armor(GET_AC(ch) + 10), return_experience(ch), 
				   IS_CARRYING_N(ch), IS_CARRYING_W(ch), CAN_CARRY_W(ch), return_alignment(GET_ALIGNMENT(ch)), 
				   GET_DEATHCOUNT(ch), doscore_end(ch)
				   );
}


ACMD(do_inventory)
{
  send_to_char(ch, "You are carrying:\r\n");
  list_obj_to_char(ch->carrying, ch, SHOW_OBJ_SHORT, TRUE);
}


ACMD(do_equipment)
{
  int i, found = 0;

  send_to_char(ch, "You are using:\r\n");
  for (i = 0; i < NUM_WEARS; i++) {
    if (GET_EQ(ch, i)) {
      if (CAN_SEE_OBJ(ch, GET_EQ(ch, i))) {
	send_to_char(ch, "%s", wear_where[i]);
	show_obj_to_char(GET_EQ(ch, i), ch, SHOW_OBJ_SHORT);
	found = TRUE;
      } else {
	send_to_char(ch, "%s", wear_where[i]);
	send_to_char(ch, "Something.\r\n");
	found = TRUE;
      }
    }
  }
  if (!found)
    send_to_char(ch, " Nothing.\r\n");
}


ACMD(do_time)
{
  const char *suf;
  int weekday, day;

  /* day in [1..35] */
  day = time_info.day + 1;

  /* 35 days in a month, 7 days a week */
  weekday = ((35 * time_info.month) + day) % 7;

  send_to_char(ch, "It is %d o'clock %s, on %s.\r\n",
	  (time_info.hours % 12 == 0) ? 12 : (time_info.hours % 12),
	  time_info.hours >= 12 ? "pm" : "am", weekdays[weekday]);

  /*
   * Peter Ajamian <peter@PAJAMIAN.DHS.ORG> supplied the following as a fix
   * for a bug introduced in the ordinal display that caused 11, 12, and 13
   * to be incorrectly displayed as 11st, 12nd, and 13rd.  Nate Winters
   * <wintersn@HOTMAIL.COM> had already submitted a fix, but it hard-coded a
   * limit on ordinal display which I want to avoid.	-dak
   */

  suf = "th";

  if (((day % 100) / 10) != 1) {
    switch (day % 10) {
    case 1:
      suf = "st";
      break;
    case 2:
      suf = "nd";
      break;
    case 3:
      suf = "rd";
      break;
    }
  }

  send_to_char(ch, "The %d%s Day of the %s, Year %d.\r\n",
	  day, suf, month_name[time_info.month], time_info.year);
}


ACMD(do_weather)
{
  const char *sky_look[] = {
    "cloudless",
    "cloudy",
    "rainy",
    "lit by flashes of lightning"
  };

  if (OUTSIDE(ch))
    {
    send_to_char(ch, "The sky is %s and %s.\r\n", sky_look[weather_info.sky],
	    weather_info.change >= 0 ? "you feel a warm wind from south" :
	     "your foot tells you bad weather is due");
    if (GET_LEVEL(ch) >= LVL_GOD)
      send_to_char(ch, "Pressure: %d (change: %d), Sky: %d (%s)\r\n",
                 weather_info.pressure,
                 weather_info.change,
                 weather_info.sky,
                 sky_look[weather_info.sky]);
    }
  else
    send_to_char(ch, "You have no feeling about the weather at all.\r\n");
}


ACMD(do_help)
{
  int chk, bot, top, mid, minlen;

  if (!ch->desc)
    return;

  skip_spaces(&argument);

  if (!*argument) {
    page_string(ch->desc, help, 0);
    return;
  }
  if (!help_table) {
    send_to_char(ch, "No help available.\r\n");
    return;
  }

  bot = 0;
  top = top_of_helpt;
  minlen = strlen(argument);

  for (;;) {
    mid = (bot + top) / 2;

    if (bot > top) {
      send_to_char(ch, "There is no help on that word.\r\n");
      mudlog(NRM, MAX(LVL_GRGOD, GET_INVIS_LEV(ch)), TRUE, "(GC) %s tried to get help on %s", GET_NAME(ch), argument);
      return;
    } else if (!(chk = strn_cmp(argument, help_table[mid].keyword, minlen))) {
      /* trace backwards to find first matching entry. Thanks Jeff Fink! */
      while ((mid > 0) &&
	 (!(chk = strn_cmp(argument, help_table[mid - 1].keyword, minlen))))
	mid--;
      page_string(ch->desc, help_table[mid].entry, 0);
      return;
    } else {
      if (chk > 0)
        bot = mid + 1;
      else
        top = mid - 1;
    }
  }
}



#define WHO_FORMAT \
"format: who [minlev[-maxlev]] [-n name] [-c classlist] [-d racelist] [-s] [-o] [-q] [-r] [-z]\r\n"

ACMD(do_who) {
  struct descriptor_data *d;
  struct char_data *tch;
  char buf[MAX_STRING_LENGTH], buf1[MAX_STRING_LENGTH];
  char name_search[MAX_NAME_LENGTH+1], arg[MAX_STRING_LENGTH];
  char mode;

  int low = 0, high = LVL_IMPL, showclass = 0, top = LVL_IMPL;
  bool who_room = FALSE, who_zone = FALSE, who_quest = 0;
  bool outlaws = FALSE, noimm = FALSE, nomort = FALSE;

  int imms = 0, morts = 0;
  size_t i;

  skip_spaces(&argument);
  strcpy(buf, argument);
  name_search[0] = '\0';

  while (*buf) {
    half_chop(buf, arg, buf1);
    if (isdigit(*arg)) {
      sscanf(arg, "%d-%d", &low, &high);
      strcpy(buf, buf1);
    } else if (*arg == '-') {
      mode = *(arg + 1);
      /* just in case; we destroy arg in the switch */
      switch (mode) {
      case 'o':
		outlaws = TRUE;
		strcpy(buf, buf1);
		break;
      case 'z':
		who_zone = TRUE;
		strcpy(buf, buf1);
		break;
      case 'q':
		who_quest = TRUE;
		strcpy(buf, buf1);
		break;
      case 'l':
		half_chop(buf1, arg, buf);
		sscanf(arg, "%d-%d", &low, &high);
		break;
      case 'n':
		half_chop(buf1, name_search, buf);
		break;
      case 'r':
		who_room = TRUE;
		strcpy(buf, buf1);
		break;
      case 'c':
		half_chop(buf1, arg, buf);
		for (i = 0; i < strlen(arg); i++)
		  showclass |= find_class_bitvector(arg);
		break;
      case 'i':
        nomort = TRUE;
        strcpy(buf, buf1);
        break;
      case 'm':
        noimm = TRUE;
        strcpy(buf, buf1);
        break;
      default:
		send_to_char(ch, WHO_FORMAT);
		return;
		break;
      }
	} else {
      send_to_char(ch, WHO_FORMAT);
      return;
    }
  }
  for (; top > 0;top--) {
    for (d = descriptor_list; d; d = d->next) {
      if (d->connected)
        continue;

	  if (GET_LEVEL(d->character) != top)  /* end of sorting by levels */
        continue;

      if (d->original)
        tch = d->original;
      else if (!(tch = d->character))
        continue;

	  if (GET_LEVEL(tch) >= LVL_IMMORT && CAN_SEE(ch, tch)) {
	    if (imms < 1)
	      send_to_char(ch, "  Immortals\r\n -=-=-=-=-=-\r\n");
	  } else if (GET_LEVEL(tch) < LVL_IMMORT && CAN_SEE(ch, tch)) {
	    if (morts < 1) {
	      if (imms > 0)
	      send_to_char(ch, "\r\n   Mortals\r\n -=-=-=-=-=-\r\n");
	      else
	        send_to_char(ch, "   Mortals\r\n -=-=-=-=-=-\r\n");
		}
	  }

      if (!CAN_SEE(ch, tch))
        continue;
      if (GET_LEVEL(tch) < low || GET_LEVEL(tch) > high)
        continue;
      if ((noimm && GET_LEVEL(tch) >= LVL_IMMORT) || (nomort && GET_LEVEL(tch) < LVL_IMMORT))
        continue;
      if (*name_search && str_cmp(GET_NAME(tch), name_search) && !strstr(GET_TITLE(tch), name_search))
        continue;
      if (outlaws && !PLR_FLAGGED(tch, PLR_KILLER) && !PLR_FLAGGED(tch, PLR_THIEF))
        continue;
      if (who_quest && !PRF_FLAGGED(tch, PRF_QUEST))
        continue;
      if (who_zone && world[ch->in_room].zone != world[tch->in_room].zone)
        continue;
      if (who_room && (tch->in_room != ch->in_room))
        continue;
      if (showclass && !(showclass & (1 << GET_CLASS(tch))))
        continue;

      if (GET_LEVEL(tch) >= LVL_IMMORT) {
		send_to_char(ch, "&y[%-13s%s&y] &w%s&y %s", "Immortal", CCNRM(ch, C_NRM),
			GET_NAME(tch), GET_TITLE(tch));
      imms++;
    } else {
      send_to_char(ch, "[%-11s] &w%s&n %s", pc_race_types[(int) GET_RACE(tch)],
		  GET_NAME(tch), GET_TITLE(tch));
      morts++;
    }

    if (GET_INVIS_LEV(tch))
      send_to_char(ch, " (i%d)", GET_INVIS_LEV(tch));
    else if (IS_AFFECTED(tch, AFF_INVISIBLE))
      send_to_char(ch, " (invis)");

    if (PLR_FLAGGED(tch, PLR_MAILING))
      send_to_char(ch, " (mailing)");
    else if (PLR_FLAGGED(tch, PLR_WRITING))
      send_to_char(ch, " (writing)");

    if (d->olc)
      send_to_char(ch, " (Creating)");
    if (PRF_FLAGGED(tch, PRF_DEAF))
      send_to_char(ch, " (deaf)");
    if (PRF_FLAGGED(tch, PRF_NOTELL))
      send_to_char(ch, " (notell)");
    if (PRF_FLAGGED(tch, PRF_NOGOSS))
      send_to_char(ch, " (nogos)");
    if (PRF_FLAGGED(tch, PRF_QUEST))
      send_to_char(ch, " (quest)");
    if (PLR_FLAGGED(tch, PLR_THIEF))
      send_to_char(ch, " (THIEF)");
    if (PLR_FLAGGED(tch, PLR_KILLER))
      send_to_char(ch, " (KILLER)");
    if (PRF_FLAGGED(tch, PRF_AFK))
      send_to_char(ch, " (AFK)");

	send_to_char(ch, "&n\r\n");
	}
  }

  send_to_char(ch, "\r\n");
  if (imms == 0)
    send_to_char(ch, "There are no visible Immortals.\r\n");
  else
    send_to_char(ch, "There %s %d visible Immortal%s.\r\n", (imms == 1 ? "is" : "are"), imms,
	(imms == 1 ? "" : "s"));
  if (morts == 0)
    send_to_char(ch, "There are no visible mortals.\r\n");
  else
    send_to_char(ch, "There %s %d visible mortal%s.\r\n", (morts == 1 ? "is" : "are"), morts,
	(morts == 1 ? "" : "s"));
  send_to_char(ch, "Total visible players: %d.\r\n", imms + morts);
}



#define USERS_FORMAT \
"format: users [-l minlevel[-maxlevel]] [-n name] [-h host] [-c classlist] [-r racelist] [-o] [-p]\r\n"

/* BIG OL' FIXME: Rewrite it all. Similar to do_who(). */
ACMD(do_users)
{
  char line[200], line2[220], idletime[10], classname[24];
  char state[30], *timeptr, mode;
  char name_search[MAX_INPUT_LENGTH], host_search[MAX_INPUT_LENGTH];
  struct char_data *tch;
  struct descriptor_data *d;
  int low = 0, high = LVL_IMPL, num_can_see = 0;
  int showclass = 0, outlaws = 0, playing = 0, deadweight = 0, showrace = 0;
  char buf[MAX_INPUT_LENGTH], arg[MAX_INPUT_LENGTH];

  host_search[0] = name_search[0] = '\0';

  strcpy(buf, argument);	/* strcpy: OK (sizeof: argument == buf) */
  while (*buf) {
    char buf1[MAX_INPUT_LENGTH];

    half_chop(buf, arg, buf1);
    if (*arg == '-') {
      mode = *(arg + 1);  /* just in case; we destroy arg in the switch */
      switch (mode) {
      case 'o':
      case 'k':
	outlaws = 1;
	playing = 1;
	strcpy(buf, buf1);	/* strcpy: OK (sizeof: buf1 == buf) */
	break;
      case 'p':
	playing = 1;
	strcpy(buf, buf1);	/* strcpy: OK (sizeof: buf1 == buf) */
	break;
      case 'd':
	deadweight = 1;
	strcpy(buf, buf1);	/* strcpy: OK (sizeof: buf1 == buf) */
	break;
      case 'l':
	playing = 1;
	half_chop(buf1, arg, buf);
	sscanf(arg, "%d-%d", &low, &high);
	break;
      case 'n':
	playing = 1;
	half_chop(buf1, name_search, buf);
	break;
      case 'h':
	playing = 1;
	half_chop(buf1, host_search, buf);
	break;
      case 'c':
	playing = 1;
	half_chop(buf1, arg, buf);
	showclass = find_class_bitvector(arg);
	break;
      case 'r':
	playing = 1;
	half_chop(buf1, arg, buf);
	showrace = find_race_bitvector(arg);
	break;
      default:
	send_to_char(ch, "%s", USERS_FORMAT);
	return;
      }				/* end of switch */

    } else {			/* endif */
      send_to_char(ch, "%s", USERS_FORMAT);
      return;
    }
  }				/* end while (parser) */
  send_to_char(ch,
	 "Num Class  Race   Name         State          Idl Login@   Site\r\n"
	 "--- ------------- ------------ -------------- --- -------- ------------------------\r\n");

  one_argument(argument, arg);

  for (d = descriptor_list; d; d = d->next) {
    if (!IS_PLAYING(d) && playing)
      continue;
    if (!IS_PLAYING(d) && deadweight)
      continue;
    if (IS_PLAYING(d)) {
      if (d->original)
	tch = d->original;
      else if (!(tch = d->character))
	continue;

      if (*host_search && !strstr(d->host, host_search))
	continue;
      if (*name_search && str_cmp(GET_NAME(tch), name_search))
	continue;
      if (!CAN_SEE(ch, tch) || GET_LEVEL(tch) < low || GET_LEVEL(tch) > high)
	continue;
      if (outlaws && !PLR_FLAGGED(tch, PLR_KILLER) &&
	  !PLR_FLAGGED(tch, PLR_THIEF))
	continue;
      if (showclass && !(showclass & (1 << GET_CLASS(tch))))
	continue;
      if (showrace && !(showrace & (1 << GET_RACE(tch))))
	continue;
      if (GET_INVIS_LEV(ch) > GET_LEVEL(ch))
	continue;

      if (d->original)
	sprintf(classname, "[%2d %s %s]", GET_LEVEL(d->original),
		CLASS_ABBR(d->original), RACE_ABBR(d->character));
      else
	sprintf(classname, "[%2d %s %s]", GET_LEVEL(d->character),
		CLASS_ABBR(d->character), RACE_ABBR(d->character));
    } else
      strcpy(classname, "   -   ");

    timeptr = asctime(localtime(&d->login_time));
    timeptr += 11;
    *(timeptr + 8) = '\0';

    if (STATE(d) == CON_PLAYING && d->original)
      strcpy(state, "Switched");
    else
      strcpy(state, connected_types[STATE(d)]);

    if (d->character && STATE(d) == CON_PLAYING && GET_LEVEL(d->character) < LVL_GOD)
      sprintf(idletime, "%3d", d->character->char_specials.timer *
	      SECS_PER_MUD_HOUR / SECS_PER_REAL_MIN);
    else
      strcpy(idletime, "");

    sprintf(line, "%3d %-11s %-12s %-14s %-3s %-8s ", d->desc_num, classname,
	d->original && d->original->player.name ? d->original->player.name :
	d->character && d->character->player.name ? d->character->player.name :
	"UNDEFINED",
	state, idletime, timeptr);

    if (d->host && *d->host)
      sprintf(line + strlen(line), "[%s]\r\n", d->host);
    else
      strcat(line, "[Hostname unknown]\r\n");

    if (STATE(d) != CON_PLAYING) {
      sprintf(line2, "%s%s%s", CCGRN(ch, C_SPR), line, CCNRM(ch, C_SPR));
      strcpy(line, line2);
    }
    if (STATE(d) != CON_PLAYING ||
		(STATE(d) == CON_PLAYING && CAN_SEE(ch, d->character))) {
      send_to_char(ch, "%s", line);
      num_can_see++;
    }
  }

  send_to_char(ch, "\r\n%d visible sockets connected.\r\n", num_can_see);
}


/* Generic page_string function for displaying text */
ACMD(do_gen_ps)
{
  switch (subcmd) {
  case SCMD_CREDITS:
    page_string(ch->desc, credits, 0);
    break;
  case SCMD_NEWS:
    page_string(ch->desc, news, 0);
    break;
  case SCMD_INFO:
    page_string(ch->desc, info, 0);
    break;
  case SCMD_WIZLIST:
    page_string(ch->desc, wizlist, 0);
    break;
  case SCMD_IMMLIST:
    page_string(ch->desc, immlist, 0);
    break;
  case SCMD_HANDBOOK:
    page_string(ch->desc, handbook, 0);
    break;
  case SCMD_POLICIES:
    page_string(ch->desc, policies, 0);
    break;
  case SCMD_MOTD:
    page_string(ch->desc, motd, 0);
    break;
  case SCMD_IMOTD:
    page_string(ch->desc, imotd, 0);
    break;
  case SCMD_CLEAR:
    send_to_char(ch, "\033[H\033[J");
    break;
  case SCMD_VERSION:
    send_to_char(ch, "%s\r\n", circlemud_version);
	send_to_char(ch, "%s\r\n", oasisolc_version);
    send_to_char(ch, "%s\r\n", DG_SCRIPT_VERSION);
    break;
  case SCMD_WHOAMI:
    send_to_char(ch, "%s\r\n", GET_NAME(ch));
    break;
  default:
    log("SYSERR: Unhandled case in do_gen_ps. (%d)", subcmd);
    /*  SYSERR_DESC:
     *  General page string function for such things as 'credits', 'news',
     *  'wizlist', 'clear', 'version'.  This occurs when a call is made to
     *  this routine that is not one of the predefined calls.  To correct
     *  it, either a case needs to be added into the function to account for
     *  the subcmd that is being passed to it, or the call to the function
     *  needs to have the correct subcmd put into place.
     */
    return;
  }
}


void perform_mortal_where(struct char_data *ch, char *arg)
{
  struct char_data *i;
  struct descriptor_data *d;

  if (!*arg) {
    send_to_char(ch, "Players in your Zone\r\n--------------------\r\n");
    for (d = descriptor_list; d; d = d->next) {
      if (STATE(d) != CON_PLAYING || d->character == ch)
	continue;
      if ((i = (d->original ? d->original : d->character)) == NULL)
	continue;
      if (IN_ROOM(i) == NOWHERE || !CAN_SEE(ch, i))
	continue;
      if (world[IN_ROOM(ch)].zone != world[IN_ROOM(i)].zone)
	continue;
      send_to_char(ch, "%-20s - %s\r\n", CAP(display_short_name(ch, d->character)), world[IN_ROOM(i)].name);
    }
  } else {			/* print only FIRST char, not all. */
    for (i = character_list; i; i = i->next) {
      if (IN_ROOM(i) == NOWHERE || i == ch)
	continue;
      if (!CAN_SEE(ch, i) || world[IN_ROOM(i)].zone != world[IN_ROOM(ch)].zone)
	continue;
      if (!isname(arg, i->player.name))
	continue;
      send_to_char(ch, "%-25s - %s\r\n", display_short_name(ch, i), world[IN_ROOM(i)].name);
      return;
    }
    send_to_char(ch, "Nobody around by that name.\r\n");
  }
}


void print_object_location(int num, struct obj_data *obj, struct char_data *ch,
			        int recur)
{
  if (num > 0)
    send_to_char(ch, "O%3d. %-25s - ", num, obj->short_description);
  else
    send_to_char(ch, "%33s", " - ");

  if (IN_ROOM(obj) != NOWHERE)
    send_to_char(ch, "[%5d] %s\r\n", GET_ROOM_VNUM(IN_ROOM(obj)), world[IN_ROOM(obj)].name);
  else if (obj->carried_by)
    send_to_char(ch, "carried by %s\r\n", PERS(obj->carried_by, ch));
  else if (obj->worn_by)
    send_to_char(ch, "worn by %s\r\n", PERS(obj->worn_by, ch));
  else if (obj->in_obj) {
    send_to_char(ch, "inside %s%s\r\n", obj->in_obj->short_description, (recur ? ", which is" : " "));
    if (recur)
      print_object_location(0, obj->in_obj, ch, recur);
  } else
    send_to_char(ch, "in an unknown location\r\n");
}



void perform_immort_where(struct char_data *ch, char *arg)
{
  struct char_data *i;
  struct obj_data *k;
  struct descriptor_data *d;
  int num = 0, found = 0;

  if (!*arg) {
    send_to_char(ch, "Players\r\n-------\r\n");
    for (d = descriptor_list; d; d = d->next)
      if (STATE(d) == CON_PLAYING) {
	i = (d->original ? d->original : d->character);
	if (i && CAN_SEE(ch, i) && (IN_ROOM(i) != NOWHERE)) {
	  if (d->original)
	    send_to_char(ch, "%-20s - [%5d] %s (in %s)\r\n",
		GET_NAME(i), GET_ROOM_VNUM(IN_ROOM(d->character)),
		world[IN_ROOM(d->character)].name, GET_NAME(d->character));
	  else
	    send_to_char(ch, "%-20s - [%5d] %s\r\n", GET_NAME(i), GET_ROOM_VNUM(IN_ROOM(i)), world[IN_ROOM(i)].name);
	}
      }
  } else {
    for (i = character_list; i; i = i->next)
      if (CAN_SEE(ch, i) && IN_ROOM(i) != NOWHERE && isname(arg, i->player.name)) {
	found = 1;
	send_to_char(ch, "M%3d. %-25s - [%5d] %s\r\n", ++num, GET_NAME(i),
		GET_ROOM_VNUM(IN_ROOM(i)), world[IN_ROOM(i)].name);
      }
    for (num = 0, k = object_list; k; k = k->next)
      if (CAN_SEE_OBJ(ch, k) && isname(arg, k->name)) {
	found = 1;
	print_object_location(++num, k, ch, TRUE);
      }
    if (!found)
      send_to_char(ch, "Couldn't find any such thing.\r\n");
  }
}



ACMD(do_where)
{
  char arg[MAX_INPUT_LENGTH];

  one_argument(argument, arg);

  if (GET_LEVEL(ch) >= LVL_IMMORT)
    perform_immort_where(ch, arg);
  else
    perform_mortal_where(ch, arg);
}



ACMD(do_levels)
{
  char buf[MAX_STRING_LENGTH];
  size_t i, len = 0, nlen;

  if (IS_NPC(ch)) {
    send_to_char(ch, "You ain't nothin' but a hound-dog.\r\n");
    return;
  }

  for (i = 1; i < LVL_IMMORT; i++) {
    nlen = snprintf(buf + len, sizeof(buf) - len, "[%2d] %8d-%-8d : ", i,
		level_exp(GET_CLASS(ch), i, GET_RACE(ch)), level_exp(GET_CLASS(ch), i + 1, GET_RACE(ch)) - 1);
    if (len + nlen >= sizeof(buf) || nlen < 0)
      break;
    len += nlen;

    nlen = snprintf(buf + len, sizeof(buf) - len, "%s\r\n", pc_class_types[GET_CLASS(ch)]);
   
    if (len + nlen >= sizeof(buf) || nlen < 0)
      break;
    len += nlen;
  }

  if (len < sizeof(buf))
    snprintf(buf + len, sizeof(buf) - len, "[%2d] %8d          : Immortality\r\n",
		LVL_IMMORT, level_exp(GET_CLASS(ch), LVL_IMMORT, GET_RACE(ch)));
  page_string(ch->desc, buf, TRUE);
}



ACMD(do_consider)
{
  char buf[MAX_INPUT_LENGTH];
  struct char_data *victim;
  int diff;

  one_argument(argument, buf);

  if (!(victim = get_char_vis(ch, buf, NULL, FIND_CHAR_ROOM))) {
    send_to_char(ch, "Consider killing who?\r\n");
    return;
  }
  if (victim == ch) {
    send_to_char(ch, "Easy!  Very easy indeed!\r\n");
    return;
  }
  if (!IS_NPC(victim)) {
    send_to_char(ch, "Would you like to borrow a cross and a shovel?\r\n");
    return;
  }
  diff = (GET_LEVEL(victim) - GET_LEVEL(ch));

  if (diff <= -10)
    send_to_char(ch, "Now where did that chicken go?\r\n");
  else if (diff <= -5)
    send_to_char(ch, "You could do it with a needle!\r\n");
  else if (diff <= -2)
    send_to_char(ch, "Easy.\r\n");
  else if (diff <= -1)
    send_to_char(ch, "Fairly easy.\r\n");
  else if (diff == 0)
    send_to_char(ch, "The perfect match!\r\n");
  else if (diff <= 1)
    send_to_char(ch, "You would need some luck!\r\n");
  else if (diff <= 2)
    send_to_char(ch, "You would need a lot of luck!\r\n");
  else if (diff <= 3)
    send_to_char(ch, "You would need a lot of luck and great equipment!\r\n");
  else if (diff <= 5)
    send_to_char(ch, "Do you feel lucky, punk?\r\n");
  else if (diff <= 10)
    send_to_char(ch, "Are you mad!?\r\n");
  else if (diff <= 100)
    send_to_char(ch, "You ARE mad!\r\n");
}



ACMD(do_diagnose)
{
  char buf[MAX_INPUT_LENGTH];
  struct char_data *vict;

  one_argument(argument, buf);

  if (*buf) {
    if (!(vict = get_char_vis(ch, buf, NULL, FIND_CHAR_ROOM)))
      send_to_char(ch, "%s", CONFIG_NOPERSON);
    else
      diag_char_to_char(vict, ch);
  } else {
    if (FIGHTING(ch))
      diag_char_to_char(FIGHTING(ch), ch);
    else
      send_to_char(ch, "Diagnose who?\r\n");
  }
}


const char *ctypes[] = {
  "off", "sparse", "normal", "complete", "\n"
};

ACMD(do_color)
{
  char arg[MAX_INPUT_LENGTH];
  int tp;

  if (IS_NPC(ch))
    return;

  one_argument(argument, arg);

  if (!*arg) {
    send_to_char(ch, "Your current color level is %s.\r\n", ctypes[COLOR_LEV(ch)]);
    return;
  }
  if (((tp = search_block(arg, ctypes, FALSE)) == -1)) {
    send_to_char(ch, "Usage: color { Off | Sparse | Normal | Complete }\r\n");
    return;
  }
  REMOVE_BIT_AR(PRF_FLAGS(ch), PRF_COLOR_1);
  REMOVE_BIT_AR(PRF_FLAGS(ch), PRF_COLOR_2);
  SET_BIT_AR(PRF_FLAGS(ch), (PRF_COLOR_1 * (tp & 1)));
  SET_BIT_AR(PRF_FLAGS(ch), (PRF_COLOR_2 * (tp & 2) >> 1));

  send_to_char(ch, "Your %scolor%s is now %s.\r\n", CCRED(ch, C_SPR), CCNRM(ch, C_OFF), ctypes[tp]);
}


ACMD(do_toggle)
{
  char buf2[4];

  if (IS_NPC(ch))
    return;

  if (GET_WIMP_LEV(ch) == 0)
    strcpy(buf2, "OFF");	/* strcpy: OK */
  else
    sprintf(buf2, "%-3.3d", GET_WIMP_LEV(ch));	/* sprintf: OK */

  if (GET_LEVEL(ch) >= LVL_IMMORT) 
    send_to_char(ch,
      "      Buildwalk: %-3s    "
      "   Clear Screen: %-3s    "
	  "    Color Level: %s\r\n"
       
	  "      No Hassle: %-3s    "
	  "      Holylight: %-3s    "
	  "     Room Flags: %-3s\r\n",

	  ONOFF(PRF_FLAGGED(ch, PRF_BUILDWALK)),
      ONOFF(PRF_FLAGGED(ch, PRF_CLS)),
      ctypes[COLOR_LEV(ch)],

 	  ONOFF(PRF_FLAGGED(ch, PRF_NOHASSLE)),
	  ONOFF(PRF_FLAGGED(ch, PRF_HOLYLIGHT)),
	  ONOFF(PRF_FLAGGED(ch, PRF_ROOMFLAGS)));
  
  send_to_char(ch,
	  "Hit Pnt Display: %-3s    "
	  "     Brief Mode: %-3s    "
	  " Summon Protect: %-3s\r\n"

	  "   Move Display: %-3s    "
	  "   Compact Mode: %-3s    "
	  "       On Quest: %-3s\r\n"

	  "   Mana Display: %-3s    "
	  "         NoTell: %-3s    "
	  "   Repeat Comm.: %-3s\r\n"

	  " Auto Show Exit: %-3s    "
	  "           Deaf: %-3s    "
	  "     Wimp Level: %-3s\r\n"

	  " Gossip Channel: %-3s    "
	  "  Grats Channel: %-3s\r\n",

	  ONOFF(PRF_FLAGGED(ch, PRF_DISPHP)),
	  ONOFF(PRF_FLAGGED(ch, PRF_BRIEF)),
	  ONOFF(!PRF_FLAGGED(ch, PRF_SUMMONABLE)),

	  ONOFF(PRF_FLAGGED(ch, PRF_DISPMOVE)),
	  ONOFF(PRF_FLAGGED(ch, PRF_COMPACT)),
	  YESNO(PRF_FLAGGED(ch, PRF_QUEST)),

	  ONOFF(PRF_FLAGGED(ch, PRF_DISPMANA)),
	  ONOFF(PRF_FLAGGED(ch, PRF_NOTELL)),
	  YESNO(!PRF_FLAGGED(ch, PRF_NOREPEAT)),

	  ONOFF(PRF_FLAGGED(ch, PRF_AUTOEXIT)),
	  YESNO(PRF_FLAGGED(ch, PRF_DEAF)),
	  buf2,

	  ONOFF(!PRF_FLAGGED(ch, PRF_NOGOSS)),
	  ONOFF(!PRF_FLAGGED(ch, PRF_NOGRATZ)));

  if (GET_LEVEL(ch) < LVL_IMMORT)
	send_to_char(ch, "    Color Level: %s\r\n", ctypes[COLOR_LEV(ch)]);

}


int sort_commands_helper(const void *a, const void *b)
{
  return strcmp(complete_cmd_info[*(const int *)a].sort_as, 
                complete_cmd_info[*(const int *)b].sort_as);
}


void sort_commands(void)
{
  int a, num_of_cmds = 0;

  while (complete_cmd_info[num_of_cmds].command[0] != '\n')
    num_of_cmds++;
  num_of_cmds++;	/* \n */

  CREATE(cmd_sort_info, int, num_of_cmds);

  for (a = 0; a < num_of_cmds; a++)
    cmd_sort_info[a] = a;

  /* Don't sort the RESERVED or \n entries. */
  qsort(cmd_sort_info + 1, num_of_cmds - 2, sizeof(int), sort_commands_helper);
}


ACMD(do_commands)
{
  int no, i, cmd_num;
  int wizhelp = 0, socials = 0;
  struct char_data *vict;
  char arg[MAX_INPUT_LENGTH];

  one_argument(argument, arg);

  if (*arg) {
    if (!(vict = get_char_vis(ch, arg, NULL, FIND_CHAR_WORLD)) || IS_NPC(vict)) {
      send_to_char(ch, "Who is that?\r\n");
      return;
    }
    if (GET_LEVEL(ch) < GET_LEVEL(vict)) {
      send_to_char(ch, "You can't see the commands of people above your level.\r\n");
      return;
    }
  } else
    vict = ch;

  if (subcmd == SCMD_SOCIALS)
    socials = 1;
  else if (subcmd == SCMD_WIZHELP)
    wizhelp = 1;

  send_to_char(ch, "The following %s%s are available to %s:\r\n",
	  wizhelp ? "privileged " : "",
	  socials ? "socials" : "commands",
	  vict == ch ? "you" : GET_NAME(vict));

  /* cmd_num starts at 1, not 0, to remove 'RESERVED' */
  for (no = 1, cmd_num = 1; complete_cmd_info[cmd_sort_info[cmd_num]].command[0] != '\n'; cmd_num++) {
    i = cmd_sort_info[cmd_num];

	/* workaround */
    if (complete_cmd_info[i].command_pointer == do_oasis && GET_LEVEL(ch) < CONFIG_LVL_BUILDER)
	  continue;
	  
	if (complete_cmd_info[i].command_pointer == do_dgscripts && GET_LEVEL(ch) < CONFIG_LVL_DGSCRIPT)
	  continue;

	if (complete_cmd_info[i].command_pointer == do_ban && GET_LEVEL(ch) < CONFIG_LVL_BAN)
	  continue;

    if (complete_cmd_info[i].command_pointer == do_gen_tog && complete_cmd_info[i].subcmd == SCMD_BUILDWALK && GET_LEVEL(ch) < CONFIG_LVL_BUILDER)
      continue;

	if (complete_cmd_info[i].command_pointer == do_wizutil && (complete_cmd_info[i].subcmd == SCMD_THAW || complete_cmd_info[i].subcmd == SCMD_FREEZE) && GET_LEVEL(ch) < CONFIG_LVL_FREEZE)
      continue;
	/* end workaround */
	
	if (complete_cmd_info[i].minimum_level < 0 || GET_LEVEL(vict) < complete_cmd_info[i].minimum_level)
      continue;

    if ((complete_cmd_info[i].minimum_level >= LVL_IMMORT) != wizhelp)
      continue;

    if (!wizhelp && socials != (complete_cmd_info[i].command_pointer == do_action || complete_cmd_info[i].command_pointer == do_insult))
      continue;

    send_to_char(ch, "%-11s%s", complete_cmd_info[i].command, no++ % 7 == 0 ? "\r\n" : "");
  }

  if (no % 7 != 1)
    send_to_char(ch, "\r\n");
}

ACMD(do_areas)
{
  int i;
  
  /****************************************************************************/
  /** Store the header for the zone listing.                                 **/
  /****************************************************************************/
  send_to_char(ch,  "       &y _                                    _     &n\r\n"
                    "       &y(@) ________________________________ (@)    &n\r\n"
				    "       &y|-|/                                \\|-|    &n\r\n"
				    "       &y|-|       &cZones in PlaniaMUD:&y        |-|&n\r\n" 
				    "       &y|-| ________________________________ |-|    &n\r\n"
				    "       &y(@)/                                \\(@)    &n\r\n"
				    "       &y ~                                    ~     &n\r\n\r\n");
  
  for (i = 0; i <= top_of_zone_table; i++)
    send_to_char(ch, "&c%-38.38s&y Build by: &w%-14s&n\r\n",
		  zone_table[i].name, zone_table[i].builders ? zone_table[i].builders : "None.");
}

