/* ************************************************************************
*   File: races.c                                  Extension of CircleMUD *
*  Usage: Source file for race-specific code                              *
* Author: Brian Williams <bmw@efn.org>                                    *
*                                                                         *
*  All rights reserved.  See license.doc for complete information.        *
*                                                                         *
*  Copyright (C) 1993, 94 by the Trustees of the Johns Hopkins University *
*  CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.               *
************************************************************************ */

#include "conf.h"
#include "sysdep.h"


#include "structs.h"
#include "db.h"
#include "utils.h"
#include "spells.h"
#include "interpreter.h"

/* Local functions */
const char *race_abbrevs[];
const char *pc_race_types[];
const char *race_menu;
const char *subrace_menu[];
int mainparse_race(char arg);
bitvector_t find_race_bitvector(const char *arg);
int invalid_race(struct char_data *ch, struct obj_data *obj);
int parse_race(char arg);

/* Not quite race related, but who cares.. */

/* The menu for choosing a caste in interpreter.c: */ 
const char *caste_menu =                              
"\r\n"                                               
"Of what caste is your family?\r\n"
"A weaker genetic influence on your character's tendencies lies in his or her\r\n"
"parentage. Depending on the profession of your forebearers, your character may\r\n"
"excel in some areas of ability more than others.\r\n\r\n"
"Select a caste: (use a capital letter to retrieve more info about the specified caste)\r\n"                                 
"&w  [&Ya&w]  Ambassador        [&Yb&w]  Artisan            [&Yc&w]  Brigand\r\n"                                      
"&w  [&Yd&w]  Courtesan         [&Ye&w]  Entertainer        [&Yf&w]  Merchant\r\n"
"&w  [&Yg&w]  Monk              [&Yh&w]  Nobleman           [&Yi&w]  Nomad\r\n"
"&w  [&Yj&w]  Politician        [&Yk&w]  Scholar            [&Yl&w]  Shaman\r\n"
"&w  [&Ym&w]  Slave             [&Yn&w]  Soldier            [&Yo&w]  Spiritualist\r\n"
"&w  [&Yp&w]  Thief             [&Yq&w]  Tradesman          [&Yr&w]  Warrior\r\n"
"&w  [&Ys&w]  You never knew\r\n&n";

const char *caste_descs[] = {
"\r\n"
"&wAMBASSADOR [&Ya&w]&n\r\n"
"\r\n"
"A person who spend most of his life in an other country on this world.\r\n"
"Representing his own nation, he often discussed and negosiated with\r\n"
"important and powerfull persons throughout the planet.\r\n"
"\r\n",
"\r\n"
"&wARTISAN [&Yb&w]&n\r\n"
"\r\n"
"A person that was well skilled in making all kinds of furniture. His\r\n"
"shop was located in a small village, where supplied the locals with\r\n"
"chairs, tables, and other wooden objects that are used in houses.\r\n"
"\r\n",
"\r\n"
"&wBRIGAND [&Yc&w]&n\r\n"
"\r\n"
"A brigand, or better known as a robber. He earned his income by robbing\r\n"
"money and other valueables from houses, and from people that came where\r\n"
"they shouldn't have.\r\n"
"\r\n",
"\r\n"
"&wCOURTESAN [&Yd&w]&n\r\n"
"\r\n"
"A man or woman that offered her body in the higher classes of the country.\r\n"
"\r\n",
"\r\n"
"&wENTERTAINER [&Ye&w]&n\r\n"
"\r\n"
"The name says it all, a person who entertains people with stories and\r\n"
"performances. A much seen guest in inns and other busy places, he knows\r\n"
"most people well, and can judge them by only a short impression.\r\n"
"\r\n",
"\r\n"
"&wMERCHANT [&Yf&w]&n\r\n"
"\r\n"
"A person that trades, buys and sells all kinds of goods, with only one goal:\r\n"
"profit. Merchants are often pretty intelligent, making the unwary buyer often\r\n"
"spend more money then they should have.\r\n"
"\r\n",
"\r\n"
"&wMONK [&Yg&w]&n\r\n"
"\r\n"
"A religious person devoted to a god. This person spend most of his life serving,\r\n"
"but later on he was promoted, and was the highest ranked monk in the castle.\r\n"
"\r\n",
"\r\n"
"&wNOBLEMAN [&Yh&w]&n\r\n"
"\r\n"
"A nobleman, this person was a ruler of a small amount of land and was devoted to\r\n"
"the king. He was pretty wealthy too, and never knew hunger or thirst.\r\n"
"\r\n",
"\r\n"
"&wNOMAD [&Yi&w]&n\r\n"
"\r\n"
"Living from what nature gives, the nomad never stays at one place for long. Used\r\n"
"to tough conditions, the nomads are strong and flexible people, but stuborn.\r\n"
"\r\n",
"\r\n"
"&wPOLITICIAN [&Yj&w]&n\r\n"
"\r\n"
"A politician is a person that has a power over a large amount of people. A politician\r\n"
"can choose for him or herself how he or she uses this influence. Politicians were often\r\n"
"hated among the lower classes.\r\n"
"\r\n",
"\r\n"
"&wSCHOLAR [&Yk&w]&n\r\n"
"\r\n"
"A man full of knowledge. Sometimes useless knowledge, but all kings have at least one\r\n"
"scholar at their service, so it leaves no doubt scholars are valuable persons in some\r\n"
"aspects.\r\n"
"\r\n",
"\r\n"
"&wSHAMAN [&Yl&w]&n\r\n"
"\r\n"
"A shaman is a person who attempts to control magic for their own purpose. While some\r\n"
"shamans have never casted a spell, there are some that are capable of great magical\r\n"
"powers. Some of the largers villages have a shaman, cities have magicians.\r\n" 
"\r\n",
"\r\n"
"&wSLAVE [&Ym&w]&n\r\n"
"\r\n"
"A slave, someone who does not have the liberty to do whatever he or she likes. While\r\n"
"slaves are treated well, most slaves are badly treated and die quickly. Slaves are\r\n"
"often convicted criminals.\r\n"
"\r\n",
"\r\n"
"&wSOLDIER [&Yn&w]\r\n"
"\r\n"
"A man skilled in melee combat. Femiliar with most modern weapons such as swords, spears\r\n"
"and daggers, a soldier is trained to kill, and to follow their masters will without\r\n"
"hesistation.\r\n"
"\r\n",
"\r\n"
"&wSPIRITUALIST [&Yh&w]&n\r\n"
"\r\n"
"A man or woman who is capable of communicating with ghosts and spirits to some extend.\r\n"
"While most spiritualists are pretending, there are some that are truelly capable of\r\n"
"talking the undead. A spiritualist is always good aligned.\r\n"
"\r\n",
"\r\n"
"&wTHIEF [&Yp&w]&n\r\n"
"\r\n"
"Living from property of others, a thiefs life consists of lies and deceit. Some thiefs\r\n"
"are organised however, and are to a certain extent loyal to a certain group or person,\r\n"
"such as a beloved king, or a general with much money to spend.\r\n"
"\r\n",
"\r\n"
"&wTRADESMAN [&Yq&w]&n\r\n"
"\r\n"
"There are many similarities between a tradesman and a merchant, but a very important\r\n"
"one is the aspect of profit. A tradesman mainly tries to make a fair living, a merchant\r\n"
"tries to gain money, at almost any cost.\r\n"
"\r\n",
"\r\n"
"&wWARRIOR [&Yr&w]&n\r\n"
"\r\n"
"A man skilled in most weapons, including ranged weapons such as longbows and crossbows.\r\n"
"A warrior fights for his own free will, and not for money nor his ruler, unlike the more\r\n"
"common soldier.\r\n"
"\r\n",
"\r\n"
"&wYOU NEVER KNEW [&Ys&w]&n\r\n"
"\r\n"
"Just like it says. You never knew your parents.\r\n"
"\r\n"
};

int parse_caste(char arg)
{
  arg = LOWER(arg);  

  switch (arg) {
    case 'a': return CASTE_AMBASSADOR;
    case 'b': return CASTE_ARTISAN;
    case 'c': return CASTE_BRIGAND;
    case 'd': return CASTE_COURTESAN;
    case 'e': return CASTE_ENTERTAINER;
	case 'f': return CASTE_MERCHANT;
	case 'g': return CASTE_MONK;
	case 'h': return CASTE_NOBLEMAN;
	case 'i': return CASTE_NOMAD;
	case 'j': return CASTE_POLITICIAN;
	case 'k': return CASTE_SCHOLAR;
	case 'l': return CASTE_SHAMAN;
	case 'm': return CASTE_SLAVE;
	case 'n': return CASTE_SOLDIER;
	case 'o': return CASTE_SPIRITUALIST;
	case 'p': return CASTE_THIEF;
	case 'q': return CASTE_TRADESMAN;
	case 'r': return CASTE_WARRIOR;
	case 's': return CASTE_UNKNOWN;
    default : return CASTE_UNDEFINED;
  }
}

void process_caste(int caste, struct descriptor_data *d)
{
  switch (caste) {
  case CASTE_AMBASSADOR:
    d->character->real_abils.cha += 2;
	d->character->real_abils.wis -= 1;
    break;
  case CASTE_ARTISAN:
	d->character->real_abils.dex += 1;
	d->character->real_abils.intel -= 1;
	break;
  case CASTE_BRIGAND:
	d->character->real_abils.con += 1;
	d->character->real_abils.intel -= 1;
	break;
  case CASTE_COURTESAN:
	d->character->real_abils.cha += 1;
	break;
  case CASTE_ENTERTAINER:
	d->character->real_abils.cha += 2;
	d->character->real_abils.str -= 1;
	break;
  case CASTE_MERCHANT:
	d->character->real_abils.intel += 1;
	d->character->real_abils.dex -= 1;
	break;
  case CASTE_MONK:
	d->character->real_abils.cha += 2;
	d->character->real_abils.str -= 1;
	break;
  case CASTE_NOBLEMAN:
	d->character->real_abils.cha += 2;
	d->character->real_abils.con -= 1;
	break;
  case CASTE_NOMAD:
	d->character->real_abils.con += 1;
	d->character->real_abils.dex += 1;
	d->character->real_abils.cha -= 1;
	d->character->real_abils.wis -= 1;
	break;
  case CASTE_POLITICIAN:
	d->character->real_abils.intel += 1;
	d->character->real_abils.str -= 1;
	break;
  case CASTE_SCHOLAR:
	d->character->real_abils.wis += 1;
	d->character->real_abils.dex -= 1;
	break;
  case CASTE_SHAMAN:
	d->character->real_abils.wis += 1;
	d->character->real_abils.cha -= 2;
	break;
  case CASTE_SLAVE:
	d->character->real_abils.con += 1;
	d->character->real_abils.wis -= 1;
	break;    
  case CASTE_SOLDIER:
	d->character->real_abils.str += 1;
	d->character->real_abils.intel -= 1;
	break;
  case CASTE_SPIRITUALIST:
    d->character->real_abils.intel += 1;
	d->character->real_abils.str -= 1;
	break;
  case CASTE_THIEF:
	d->character->real_abils.con += 1;
	d->character->real_abils.wis -= 1;
	break;
  case CASTE_TRADESMAN:
	d->character->real_abils.cha += 2;
	break;
  case CASTE_WARRIOR:
	d->character->real_abils.str += 2;
	d->character->real_abils.wis -= 1;
	d->character->real_abils.cha -= 1;
	break;
  case CASTE_UNKNOWN:
	switch(rand_number(0, 5)) {
	case 0: d->character->real_abils.str += 1; break;
	case 1: d->character->real_abils.con += 1; break;
	case 2: d->character->real_abils.dex += 1; break;
	case 3: d->character->real_abils.intel += 1; break;
	case 4: d->character->real_abils.wis += 1; break;
	case 5: d->character->real_abils.cha += 1; break;
	}
    break;
  default: 
	/* Get rid of this freak */
	log("SYSERR: %s had undefined caste in process_caste!", GET_NAME(d->character));
	STATE(d) = CON_CLOSE;
	break;
  }
  return;
}

/* End of caste section */ 

const char *race_abbrevs[] = {
  "Hob",
  "Elf",
  "Hum",
  "Orc",
  "Mys",
  "Har",
  "Sto",
  "Fal",
  "Van",
  "Nol",
  "Tel", 
  "Ail",
  "Num",
  "Hil",
  "Eng",
  "Uru",
  "Ole",
  "Mor",
  "Cen", 
  "Min",
  "Avi",
  "\n"
};


const char *pc_race_types[] = {
  "Hobbit",
  "Elf",
  "Human",
  "Orc",
  "Mystical",
  "Harfoot",
  "Stoor",
  "Fallohide",
  "Vanyar",
  "Noldor",
  "Teleri",
  "Aiel",
  "Numeron",
  "Hildor",
  "Engwar",
  "Uruk-hai",
  "Oleg-hai",
  "Mori-hai",
  "Centaur",
  "Minotaur",
  "Avian",
  "\n"
};

const char *pc_race_types_type2[] = {  
  "Hobbit",                       
  "Elven",                         
  "Human",                       
  "Orcish",                       
  "Mystic",
  "Harfoot",
  "Stoor",
  "Fallohide",
  "Vanyar",
  "Noldor",
  "Teleri",
  "Aiel",
  "Numeron",
  "Hildor",
  "Engwar",
  "Uruk-hai",
  "Oleg-hai",
  "Mori-hai",
  "Centaur",
  "Minotaur",
  "Avian",
  "\n"
}; 




/* The menu for choosing a race in interpreter.c: */ 
const char *race_menu =                              
"\r\n"                                               
"Long ago, many races inhabited the world. They were once united, but in a time\r\n" 
"before written history a great war erupted. Many races were destroyed. Only\r\n"
"remain to this day:\r\n\r\n" 
"  &y[&na&y]&c The Hobbits&n\r\n"
"  &y[&nb&y]&c The Elves&n\r\n"
"  &y[&nc&y]&c The Humans&n\r\n"
"  &y[&nd&y]&c The Orcs&n\r\n"
"  &y[&ne&y]&c The Mystical&n\r\n\r\n"
"Even with in themselves, these fractured races could not live in peace. Civil\r\n"
"war broke out, resulting in a wound that continues to fester. Once five united\r\n"
"races, they have shattered into many. Be warned that stronger races will need\r\n"
"more efford to raise above their stations.\r\n";


const char *pc_race_descs[] = {
"\r\n"
"&cTHE HOBBITS &y[&ne&y]&n\r\n"
"\r\n"
"These small people are often underestimated because of their size. That is\r\n"
"their advantage however. Quick and agile they can hide at a moments notice,\r\n"
"unable to escape detection by even the most aware of creatures. Halflings\r\n"
"make cozy homes by burrowing into the sides of grassy hills usually taking\r\n"
"to a quit life of farming. Peace loving and lazy, they seldom leave their\r\n"
"village to roam among the 'giants', but there are young adventurous ones that\r\n"
"will lighten your purse if you don't watch out.\r\n"
"&y---------------------------------------------------------------------------------&n\r\n"
"\r\n",
"\r\n"
"&cTHE ELVES &y[&nb&y]&n\r\n"
"\r\n"
"The Elvin civil war came about shortly after the ancient war, when the elves\r\n"
"banned the use of magic due to its destructive capabilities. Many of the elves\r\n"
"did not want to give up the ancient art their forefathers had practiced.\r\n"
"Persecution of these elves took place, and many fled. They took root in the\r\n"
"valley to the south of the Elvin Kingdom, and the \"high\" elves took up charge\r\n"
"after the \"Dark\" Elves. The Dark Elves however, banded together, and using\r\n"
"powerful magic, devastated the attacking army. The High Elves didn't give up.\r\n"
"In a desperate bid, they recruited many of the most infamous rogues from their\r\n"
"dungeons, and forced them to march on the Dark Elf stronghold. But instead of\r\n"
"attacking the Dark Elves, many deserted to the surrounding forests, in time\r\n"
"becoming known as \"Sylvan\" Elves. With all their priests and mages gone, their\r\n"
"army devastated, the High Elves were forced to recognize the Dark Elves'\r\n"
"independence.\r\n" 
"&y---------------------------------------------------------------------------------&n\r\n"
"\r\n",
"\r\n"
"&cTHE HUMANS &y[&nc&y]&n\r\n"
"\r\n"
"Go to the nearest mirror and have a look.  Maybe not the finest specimen\r\n"
"but you'll suffice :)\r\n"
"Humans are the predominant race in Plania. Humans rule most of the significant\r\n"
"Empires and kingdoms in this realm. They are the most social and tolerant of\r\n"
"races, excepting perhaps the halflings.\r\n"
"&y---------------------------------------------------------------------------------&n\r\n"
"\r\n",
"\r\n"
"&cTHE ORCS &y[&nd&y]&n\r\n"
"\r\n"
"The Orks were never really unified, the Uruk Hai simply enslaved the other\r\n"
"Orkish races. During the Orkish civil war, the Oleg Hai gained their freedom\r\n"
"with the help of the Harpy race. As a result, the first major distinction\r\n"
"between the races became clear.\r\n" 
"\r\n",
"&y---------------------------------------------------------------------------------&n\r\n"
"\r\n"
"&cTHE MYSTICAL &y[&ne&y]&n\r\n"
"\r\n"
"Before the ancient war much more numerous than now. Most humans and hobbits\r\n"
"think these creatures belong in children-tales, and never existed. They are\r\n"
"wrong. Living in small groups, they often pay attention that they are not\r\n" 
"seen by wandering travelers and adventurers.\r\n"
"The three most numerous mystical creatures are Centaurs, Minotaurs, and Avians.\r\n"
"&y---------------------------------------------------------------------------------&n\r\n"
"\r\n",
"\r\n"
"&cTHE HARFOOT HOBBIT &y[&na&y]&n\r\n"
"\r\n"
"Small, brown skinned. They prefer highlands and hillsides. They are by far the\r\n"
"most numerous hobbit species, and they dealt much with Dwarves in earlier times.\r\n"
"They were more apt to settle downin one place and also had more love for living\r\n"
"in holes.\r\n" 
"\r\n",
"\r\n"
"&cTHE STOOR HOBBIT &y[&nb&y]&n\r\n"
"\r\n"
"A broader and heavier hobbit, with larger hands and feet. They prefer flat lands\r\n"
"and riversides. Less shy of Men, it is not surprising that you would find much of\r\n"
"the Stoors around Bree.\r\n" 
"\r\n",
"\r\n"
"&cTHE FALLOHIDE HOBBIT &y[&nc&y]&n\r\n"
"\r\n"
"Fairer of skin and hair, and taller and slimmer than the other two breeds. Lovers\r\n"
"of trees and woodlands. Not surprisingly, more friendly with Elves and more\r\n"
"adventurous too.\r\n"
"\r\n",
"\r\n"
"&cTHE VANYAR &y[&na&y]&n\r\n"
"\r\n"
"Proud and militaristic, they see themselves as the true elven race. Not as\r\n"
"powerfull with magic as they once were, but their elves are one of most powerfull\r\n"
"fighters in the realm. They were defeated by the other two subraces shortly after\r\n"
"the ancient war and are still ashamed of this since they see them as inferior to\r\n"
"themselves. As all elves, they hate any and all evil.\r\n"
"\r\n",
"\r\n"
"&cTHE NOLDOR &y[&nb&y]&n\r\n"
"\r\n"
"These elves, also known as sylvan elves or wood-elves, are usually smart and\r\n"
"charming. They love to write songs, and prefer to remain peacefull. Unneeded\r\n"
"violence is frowned upon, but they will never step away from a fight, and are\r\n"
"dextile fighters. They use and love magic, and are serving the spirits of the\r\n"
"woods.\r\n" 
"\r\n",
"\r\n"
"&cTHE TELERI &y[&nc&y]&n\r\n"
"\r\n"
"The dark elves have fought for the freedom of use of magic. They devestated the\r\n"
"Vanyar armies thanks to this. However, the dark elves have lost a big portion of\r\n"
"their charisma and charms, as they do whatever they like. They tend more to humans\r\n"
"than they do to elves. Teleri are pretty much free, but it does have it bad sides.\r\n"
"\r\n",
"\r\n"
"&cTHE AIEL &y[&na&y]&n\r\n"
"\r\n"
"The Aiel are desert people, living as nomads in large tribes. They are dextile, and\r\n"
"don't need much water nor food. Usually tanned and with blue eyes, they are quite\r\n"
"easy to recognise.\r\n"
"\r\n",
"\r\n"
"&cTHE NUMERON &y[&nb&y]&n\r\n"
"\r\n"
"The Numeron are by far the most common humans in Plania. They like to live in large\r\n"
"cities, appreciating the company of other humanoids. They care greatly about science\r\n"
"and magic, and therefore it is not suprising powerfull mages are Numeron.\r\n"
"\r\n",
"\r\n"
"&cTHE HILDOR &y[&nc&y]&n\r\n"
"\r\n"
"In elven language, Hildor means followers, they gave them this name because the Hildor\r\n"
"follow their own heart. The hildor live in small groups in forests and large area's\r\n"
"where they live of nature. They are dextile, and often offer themselves as guides,\r\n"
"if they even can be found.\r\n"
"\r\n",
"\r\n"
"&cTHE ENGWAR &y[&nd&y]&n\r\n"
"\r\n"
"The engwar live close to the evil empires, and use dark magic. They like to live in\r\n"
"small villages, and in small camps made out of tents. They sometimes invade weaker\r\n"
"Numeron cities by suprise, and leave quickly enough to prevent counter attacks. Not\r\n"
"much is known of them.\r\n"
"\r\n",
"\r\n"
"&cTHE URUK-HAI &y[&na&y]&n\r\n"
"\r\n"
"Also known as the King Orks, their aggression and offense is unmatched. They have\r\n"
"the tendency to press on where others may not. They wish only to destroy, and often\r\n"
"get that wish.\r\n"
"\r\n",
"\r\n"
"&cTHE OLEG-HAI &y[&nb&y]&n\r\n"
"\r\n"
"Somewhat less aggressive then their cousins, the Uruk-Hai, the Oleg-Hai are less\r\n"
"concerned with destruction, and more concerned with dominance. As such, they can defend\r\n"
"things they have conquered much better. However, with all their knowledge of combat\r\n"
"techniques, they are lacking on the spiritual side.\r\n" 
"wish.\r\n"
"\r\n",
"\r\n"
"&cTHE MORI-HAI &y[&nc&y]&n\r\n"
"\r\n"
"Long thought to have perished in the ancient war, the Mori Hai were re-discovered\r\n"
"looting a small human outpost. It is thought that when all was lost, they retreated to\r\n"
"the mountains of Plania and hid themselves. The little that is known about them is\r\n"
"from the ancient documents pertaining to the war, in which they played a central part.\r\n"
"They are fast and agile, but what other secrets do they hold?\r\n"
"\r\n",
"\r\n"
"&cTHE CENTAUR &y[&na&y]&n\r\n"
"\r\n"
"Centaurs have the head and upper torso of a human and the body of a horse. Often\r\n"
"wise and intelligent, they love books, the thicker the better. Centaurs are also\r\n"
"fond of music and poetry, and, above all, peace. Although peacefull, Centaurs\r\n"
"have an advantage in battle against most creatures as they are able to use\r\n"
"hand-held weapons as well as kick with their powerful rear legs, striking enemies\r\n"
"with their hooves. (which are usually shod with iron)\r\n"
"\r\n",
"\r\n"
"&cTHE MINOTAUR &y[&nb&y]&n\r\n"
"\r\n"
"Minotaurs have their own code of honor. The one still standing is the winner.\r\n"
"These bull-like creatures are extremely strong. Even unarmed they can dismember\r\n" 
"you with their horns. Since their appearance is often intimidating enough, they\r\n"
"seldom use their wits.\r\n"
"\r\n",
"\r\n"
"&cTHE AVIAN &y[&nc&y]&n\r\n"
"\r\n"
"The term Avian describes several related races of highly magical nature. All are\r\n"
"7 foot bird like humanoids. They have both wings and hands, the wings being three\r\n"
"times as long as the Avian is tall. The head is that of a large bird of prey, with a\r\n"
"hooked black beak and large round eyes. The whole body is feathered except for the\r\n"
"hands and taloned feet. Color varies from avian to avian.\r\n"
"\r\n"
};


const char *subrace_menu[] = {
"&y---------------------------------------------------------------------------------&n\r\n\r\n"
"The Hobbits, the small but brave, there are three subraces. The Harfoots, the more\r\n" 
"'common' hobbits who usually like to live their quiet life. The Stoors, stronger\r\n"
"build hobbits that are less shy. And finally the Fallohides, lovers of the nature\r\n"
"and the adventurers.\r\n"
"Choose your subrace:  &y(&nuse a capital letter to view the description of a race&y)&n\r\n"                                 
"  &y[&na&y]  &cHarfoot&n\r\n"                                      
"  &y[&nb&y]  &cStoor&n\r\n"
"  &y[&nc&y]  &cFallohide&n\r\n"  
"\r\n",
"&y---------------------------------------------------------------------------------&n\r\n\r\n"
"The Elves, noble people, there are three subraces. The Vanyar or the high elves, are\r\n"
"noble but often arrogant. The Noldor, better known as sylvan or wood elves, they\r\n"
"are the most common, and almost always live in forests. Then there are Teleri, elves\r\n"
"that worship the dark and use magic for foul things. Vanyar and Teleri deny their\r\n"
"existance.\r\n"
"Choose your subrace:  &y(&nuse a capital letter to view the description of a race&y)&n\r\n"                                 
"  &y[&na&y]  &cVanyar&n (High elf)\r\n"                                      
"  &y[&nb&y]  &cNoldor&n (Sylvan elf)\r\n"
"  &y[&nc&y]  &cTeleri&n (Dark elf)\r\n"
"\r\n",
"&y---------------------------------------------------------------------------------&n\r\n\r\n"
"The intelligent Humans, there are four subraces. First the Aiel, these desert people\r\n"
"are extremely tough. They are sometimes distrusted because of their strange habbits.\r\n"
"The second subrace are the numeron, known as the common human. Often living in large\r\n"
"cities, they like the company of other humans. Then there are the Hildor. They love\r\n"
"nature and peace, and they are often refered to as barbarians by the numeron, but in\r\n"
"fact the opposite is true. Finally there are the Engwar. These mostly cruel people are\r\n"
"close to the dark and live in well hidden places. They use dark magic.\r\n"
"Choose your subrace:  &y(&nuse a capital letter to view the description of a race&y)&n\r\n"                                 
"  &y[&na&y]  &cAeil&n (Desert people)\r\n"                                      
"  &y[&nb&y]  &cNumeron&n (Common humans)\r\n"
"  &y[&nc&y]  &cHildor&n (Peacekeepers)\r\n"
"  &y[&nd&y]  &cEngwar&n (The darker humans)\r\n" 
"\r\n",
"&y---------------------------------------------------------------------------------&n\r\n\r\n"
"Cruel and agressive, they are species of humanoids that band together in tribes and\r\n"
"survive off hunting and raiding. The first subrace is the Uruk-hai. They are agressive\r\n"
"and only care about destroying. Then there are the Oleg-hai. Somewhat less agressive\r\n"
"then their cousins, the Uruk-hai, they care more about dominance, and defend the\r\n"
"things they conquer. The final race is the Mori-hai. Not much is known of them, other\r\n"
"then that they are nearly extincted, and they are agile and dextile.\r\n"
"are extremely tough. They are sometimes distrusted because of their strange habbits.\r\n"
"Choose your subrace:  &y(&nuse a capital letter to view the description of a race&y)&n\r\n"                                 
"  &y[&na&y]  &cUruk-hai&n\r\n"                                      
"  &y[&nb&y]  &cOleg-hai&n\r\n"
"  &y[&nc&y]  &cMori-hai&n\r\n"
"\r\n",
"&y---------------------------------------------------------------------------------&n\r\n\r\n"
"Not really being one race, there are a lot of mystical creatures in Plania. There are\r\n"
"many, and most of them are not yet discovered. You can be born as one of these three\r\n"
"races. Centaurs have the head and upper torso of a human and the body of a horse.\r\n"
"Holding wisdom above all else, they are a fairly peaceful and quiet race. Minotaurs\r\n"
"are half bull, half human. They have enourmous strength at their deposition. Then\r\n" 
"there are Avians. These creatures have huge wings, usually three times as wide as the\r\n"
"avian is tall. This grants them some unique abilities.\r\n"
"Choose your subrace:  &y(&nuse a capital letter to view the description of a race&y)&n\r\n"                                 
"  &y[&na&y]  &cCentaur&n\r\n"                                      
"  &y[&nb&y]  &cMinotaur&n\r\n"
"  &y[&nc&y]  &cAvian&n\r\n"
"\r\n"
};



int parse_mainrace(char arg)
{
  arg = LOWER(arg);  

  switch (arg) {
    case 'a': return RACE_HOBBIT;
    case 'b': return RACE_ELF;
    case 'c': return RACE_HUMAN;
    case 'd': return RACE_ORC;
    case 'e': return RACE_MYSTICAL;
    default : return RACE_UNDEFINED;
  }
}


int parse_subrace(char arg, int load_result)
{
  arg = LOWER(arg);  

  switch (load_result) {
  case RACE_HOBBIT:
	switch (arg) {
	case 'a': return RACE_HOBBIT_HARFOOT;
	case 'b': return RACE_HOBBIT_STOOR;
	case 'c': return RACE_HOBBIT_FALLOHIDE;
	default : return RACE_HOBBIT;
	}
  case RACE_ELF:
	switch (arg) {
	case 'a': return RACE_ELF_VANYAR;
	case 'b': return RACE_ELF_NOLDOR;
	case 'c': return RACE_ELF_TELERI;
	default : return RACE_ELF;
	}
  case RACE_HUMAN:
	switch (arg) {
	case 'a': return RACE_HUMAN_AIEL;
	case 'b': return RACE_HUMAN_NUMERON;
	case 'c': return RACE_HUMAN_HILDOR;
	case 'd': return RACE_HUMAN_ENGWAR;
	}
  case RACE_ORC:
    switch (arg) {
	case 'a': return RACE_ORC_URUKHAI;
	case 'b': return RACE_ORC_OLEGHAI;
	case 'c': return RACE_ORC_MORIHAI;
	default : return RACE_ORC;
	}
  case RACE_MYSTICAL:
    switch (arg) {
	case 'a': return RACE_MYSTIC_CENTAUR;
	case 'b': return RACE_MYSTIC_MINOTAUR;
	case 'c': return RACE_MYSTIC_AVIAN;
	default : return RACE_MYSTICAL;
	}
  default : return RACE_UNDEFINED;
  }
}


bitvector_t find_race_bitvector(const char *arg)
{
  size_t rpos, ret = 0;

  for (rpos = 0; rpos < strlen(arg); rpos++)
    ret |= (1 << parse_race(arg[rpos]));

  return (ret);
}

int parse_race(char arg)
{
  arg = LOWER(arg);  

  switch (arg) {
    case 'a': return RACE_HOBBIT;
    case 'b': return RACE_ELF;
    case 'c': return RACE_HUMAN;
    case 'd': return RACE_ORC;
    case 'e': return RACE_MYSTICAL;
    case 'f': return RACE_HOBBIT_HARFOOT;
    case 'g': return RACE_HOBBIT_STOOR;
    case 'h': return RACE_HOBBIT_FALLOHIDE;
    case 'i': return RACE_ELF_VANYAR;
    case 'j': return RACE_ELF_NOLDOR;
	case 'k': return RACE_ELF_TELERI;
    case 'l': return RACE_HUMAN_AIEL;
    case 'm': return RACE_HUMAN_NUMERON;
    case 'n': return RACE_HUMAN_HILDOR;
    case 'o': return RACE_HUMAN_ENGWAR;
	case 'p': return RACE_ORC_URUKHAI;
    case 'q': return RACE_ORC_OLEGHAI;
    case 'r': return RACE_ORC_MORIHAI;
    case 's': return RACE_MYSTIC_CENTAUR;
    case 't': return RACE_MYSTIC_MINOTAUR;
	case 'u': return RACE_MYSTIC_AVIAN;
 	default : return RACE_UNDEFINED;
  }
}


int invalid_race(struct char_data *ch, struct obj_data *obj)
{
  if ((OBJ_FLAGGED(obj, ITEM_ANTI_HOBBIT) && IS_HOBBIT(ch)) ||
      (OBJ_FLAGGED(obj, ITEM_ANTI_ELF) && IS_ELF(ch)) ||
      (OBJ_FLAGGED(obj, ITEM_ANTI_HUMAN) && IS_HUMAN(ch)) ||
      (OBJ_FLAGGED(obj, ITEM_ANTI_ORC) && IS_ORC(ch)) ||
      (OBJ_FLAGGED(obj, ITEM_ANTI_MYSTICAL) && IS_MYSTICAL(ch)))
    return (1);
  else
    return (0);
}

